package physics;

import physics.Constants;

/**
 * @author Boris Yartsev
 */
public class MatrixSolver {
    public static final int DONE_STATE = 0;
    public static final int NOT_DONE_STATE = 1;
    public static final int DONE_ERROR_STATE = 2;
    private double[][] matrix;
    private double [][] sym;
    private double[] vector;
    private double[] symvect;
    private double[] result;

    private int state;
    public MatrixSolver(int size) {
        matrix = new double[size][size];
        sym = new double[size][size];
        vector = new double[size];
        symvect = new double[size];
        result = new double[size];
        state = NOT_DONE_STATE;
    }

    public void loadMatrix(double [][] matrix, double vector[]) {
        state = NOT_DONE_STATE;
        for (int i = 0; i < matrix.length; i++) {
            for (int j = 0; j < matrix.length; j++) {
                this.matrix[i][j] = matrix[i][j];
            }
        }

        for (int i = 0; i < vector.length; i++) {
            this.vector[i] = vector[i];
        }

    }

    public void loadCoeffAt(double value, int i, int j) {
        matrix[i][j] = value;
        state = NOT_DONE_STATE;
    }

    public void loadVectorAt(double value, int place) {
        vector[place] = value;
        state = NOT_DONE_STATE;
    }

    private void solveMatrix() {
        int n = matrix.length;
        double[] tmp = new double[n];

        if (n == 2) {
            double det = matrix[0][0] * matrix[1][1] - matrix[0][1] * matrix[1][0];
            double det1 = vector[0] * matrix[1][1] - matrix[0][1] * vector[1];
            double det2 = -vector[0] * matrix[1][0] + matrix[0][0] * vector[1];

            result[0] = det1 / det;
            result[1] = det2 / det;
            state = DONE_STATE;
        } else {
            makeSymmetricMatrixAndVector();

            for (int i = 0; i < n; i++) {
                result[i] = vector[i];
            }

            double maxDiff = 1;
            while (true) {
                for (int i = 0; i < n; i++) {
                    tmp[i] = 0;
                    for (int j = 0; j < n; j++) {
                        tmp[i] += matrix[i][j] * vector[j];
                    }

                    if (Math.abs(tmp[i] - vector[i]) > maxDiff) {
                        maxDiff = Math.abs(tmp[i] - vector[i]);
                        vector[i] = tmp[i];
                    }
                }

                if (maxDiff < Constants.EPS) {
                    break;
                }
            }
            state = DONE_STATE;
        }
    }

    private void makeSymmetricMatrixAndVector() {
        int n = matrix.length;

        for (int i = 0; i < n; i++) {
            symvect[i] = 0;

            for (int j = 0; i < n; i++) {
                sym[i][j] = 0;
                for (int k = 0; k < n; k++) {
                    sym[i][j] += matrix[k][i] * matrix[k][j];
                }
                symvect[i] += matrix[j][i] * vector[j];
            }
        }

        for (int i = 0; i < n; i++) {
            for (int j = 0; j < n; j++) {
                matrix[i][j] = sym[i][j];
            }
            vector[i] = symvect[i];
        }
    }


    public double[] getAnswer() {
        if ((state == NOT_DONE_STATE) || (state == DONE_ERROR_STATE)) {
            solveMatrix();
        }

        if (state == DONE_STATE) {
            return result;
        } else
        if (state == DONE_ERROR_STATE) {
            return null;
        }
        return null;
    }
}
