package physics;

import robots.view.DrawableObject;
import robots.field.CoordinateObject;

/**
 * This is the class, that represents some object on the field.
 * Boris Yartsev
 */
public abstract class PhysicalObject implements CalcObject, DrawableObject, CoordinateObject {

    public static final int E100_UPDATE_COORDINATES_CALC_EVENT = -1;
    public static final int E101_RESTART_CALC_EVENT = -2;

    //Mass of the object
    protected double mass;

    //Inertia moment of the object
    protected double inertiaMoment;

    //Coordinates. x and y in range -1000 to 1000, angle in range 0 .. 2 * PI
    protected double x = 0;
    protected double y = 0;
    protected double angle = 0;

    //Speed
    protected double lSpeedX = 0;
    protected double lSpeedY = 0;
    protected double wTorque = 0;

    //Acceleration
    protected double accX = 0;
    protected double accY = 0;
    protected double accTorque = 0;

    //The characteristic size of the object
    protected double size;

    private int calcState = 0;

    public PhysicalObject(double mass, double inertiaMoment, double size) {
        this.mass = mass;
        this.inertiaMoment = inertiaMoment;
        this.size = size;
    }

    public double getMass() {
        return mass;
    }

    public double getInertiaMoment() {
        return inertiaMoment;
    }

    public void setX(double x) {
        this.x = x;
    }

    public void setY(double y) {
        this.y = y;
    }

    public void setAngle(double angle) {
        this.angle = angle;
    }

    public double getX() {
        return x;
    }

    public double getY() {
        return y;
    }

    public double getAngle() {
        return angle;
    }

    public double getlSpeedX() {
        return lSpeedX;
    }

    public double getlSpeedY() {
        return lSpeedY;
    }

    public double getwTorque() {
        return wTorque;
    }

    public double getAccX() {
        return accX;
    }

    public double getAccY() {
        return accY;
    }

    public double getAccTorque() {
        return accTorque;
    }

    public double getSize() {
        return size;
    }

    //these two methods are used for updating the calc state of the automaton
    public int getCalcState() {
        return calcState;
    }

    public void setCalcState(int calcState) {
        this.calcState = calcState;
    }

    public void updateCalcState(int calcEvent, int data, double time, Object objData) {

    }
}