package physics;

import java.awt.image.BufferedImage;
import java.awt.image.WritableRaster;
import java.awt.*;

/**
 * @author Boris Yartsev
 */
public class Utilities {
    public static double normalizeAngle02(double angle) {
        if (angle > 2 * Math.PI) {
            angle = angle % (2 * Math.PI);
        }

        if (angle < 0) {
            angle = (2 * Math.PI) - (Math.abs(angle) % (2 * Math.PI));
        }

        return angle;
    }

    public static double normalizeAngle11(double angle) {
        angle = normalizeAngle02(angle);
        if (angle > Math.PI) {
            angle -= (2 * Math.PI);
        }

        return angle;
    }

    /**
     * Converts the x, y coordinates to the polar angle
     * @param x
     * @param y
     * @return
     */
    public static double convertToAngle(double x, double y) {
        double radius = Math.sqrt(x * x + y * y);

        if (radius < Constants.EPS) {
            return 0;
        }

        double angle = Math.acos(x / radius);

        if (y < 0) {
            angle = 2 * Math.PI - angle;
        }
        return angle;
    }

    public static double convertToRadius(double x, double y) {
        return Math.sqrt(x * x + y * y);        
    }


    public static double convertCoord(double value, double x1, double x2, double xnew1, double xnew2) {
        double w1 = x2 - x1;
        double par = (value - x1) / w1;
        double w2 = xnew2 - xnew1;
        return (w2 * par + xnew1);
    }

    public static BufferedImage makeTransparentColor(BufferedImage img, Color col) {
        switch(img.getType()) {
            case BufferedImage.TYPE_INT_ARGB: {
                WritableRaster raster = img.getRaster();

                int w = raster.getWidth();
                int h = raster.getHeight();

                int [] array = new int[4];

                int r = col.getRed();
                int g = col.getGreen();
                int b = col.getBlue();

                for (int i = 0; i < w; i++) {
                    for (int j = 0; j < h; j++) {
                        raster.getPixel(i, j, array);
                        if ((array[0] == r) && (array[1] == g) && (array[2] == b)) {
                            array[3] = 0;
                            raster.setPixel(i, j, array);
                        }
                    }
                }
                break;
            }
        }

        return img;
    }

    public static int convertRad2Angle(double radian) {
        radian = normalizeAngle02(radian);
        return (int) Math.round((radian * 180) / Math.PI);
    }

    private static int id = 1;

    public synchronized static int generateUniqueId() {
        id++;
        return id;
    }
}
