package robots.comm;

import java.util.ArrayList;
import java.util.List;
import java.util.Iterator;

import physics.Utilities;
import physics.CalcObject;
import physics.PhysicalObject;

/**
 * @author Boris Yartsev
 */
public class InfraredField implements CalcObject {

    private int state = PERMIT_SIGNALS_CALC_STATE;
    private List emitters;
    private List receivers;

    //the signals are not transmitted
    public static final int PROHIBIT_SIGNALS_CALC_STATE = 1;

    //The signals are transmitted
    public static final int PERMIT_SIGNALS_CALC_STATE = 2;

    public InfraredField() {
        emitters = new ArrayList();
        receivers = new ArrayList();
    }

    public void addEmitter(InfraredEmitter emitter) {
        emitters.add(emitter);
    }

    public void addReceiver(InfraredReceiver receiver) {
        receivers.add(receiver);
    }

    public void addReceiverEmitter(InfraredReceiverEmitter irrecem) {
        addEmitter(irrecem.getEmitter());
        addReceiver(irrecem.getReceiver());
    }

    public List getEmitters() {
        return emitters;
    }

    public List getReceivers() {
        return receivers;
    }

    public synchronized void updateCalcState(int event, int data, double timeDelta, Object objectData) {
        switch(getCalcState()) {
            case PERMIT_SIGNALS_CALC_STATE: {
                switch(event) {
                    case PhysicalObject.E100_UPDATE_COORDINATES_CALC_EVENT:
                        z1_transmitSignals();
                        z3_updateIRDA(timeDelta);
                        break;
                }

                break;
            }
        }
    }

    private void z1_transmitSignals() {
        for (Iterator iter = emitters.iterator(); iter.hasNext();) {
            InfraredEmitter emitter = (InfraredEmitter) iter.next();

            if (emitter.isSignalReady()) {
                byte signal = emitter.getSendingValue();
                emitter.signalSent();

                double x = emitter.getX();
                double y = emitter.getY();
                double angle = emitter.getAngle();
                double emittingAngle = emitter.getInfraredAlpha();
                double length = emitter.getInfraredRadius();

                for (Iterator iter2 = receivers.iterator(); iter2.hasNext();) {
                    InfraredReceiver receiver = (InfraredReceiver) iter2.next();

                    if (transmitAllowed(emitter, receiver)) {
                        double recX = receiver.getX();
                        double recY = receiver.getY();
                        double recAngle = receiver.getAngle();
                        double recAlpha = receiver.getRecieveAngle();

                        if (transmitPossible(x, y, angle, emittingAngle, length,
                                recX, recY, recAngle, recAlpha)) {
                            receiver.addMessage(signal);
                        }
                    }
                }
            }
        }
    }

    private void z3_updateIRDA(double timeDelta) {
        for (Iterator iter = emitters.iterator(); iter.hasNext(); ) {
            InfraredEmitter emitter = (InfraredEmitter) iter.next();
            emitter.updateCalcState(PhysicalObject.E100_UPDATE_COORDINATES_CALC_EVENT, 0, timeDelta, null);
        }
    }

    private boolean transmitPossible(double emX, double emY, double emAngle, double emAlpha, double emRadius,
                                     double recX, double recY, double recAngle, double recAlpha) {
        double dx = recX - emX;
        double dy = recY - emY;
        double radius = Utilities.convertToRadius(dx, dy);
        double angle = Utilities.convertToAngle(dx, dy);

        if (radius > emRadius) {
            return false;
        }

        double angDelta = Utilities.normalizeAngle11(angle - emAngle);

        if (Math.abs(angDelta) > emAlpha / 2) {
            return false;
        }

        double recAngDelta = Utilities.normalizeAngle11(angle - recAngle + Math.PI);

        if (Math.abs(recAngDelta) > recAlpha / 2) {
            return false;
        }

        return true;

    }

    private boolean transmitAllowed(InfraredEmitter em, InfraredReceiver rec) {
        return true;
    }

    public int getCalcState() {
        return state;
    }

    public void setCalcState(int state) {
        this.state = state;
    }
}
