package robots.comm;

import auto.Automaton;
import auto.AutomatonUpdator;

import java.util.*;
import java.util.List;
import java.awt.*;

import robots.view.DrawableObject;
import robots.field.CoordinateObject;

/**
 * In infrared receiver we will make it possible to recieve as many signals as possible.
 * This is an assumption
 * @author Boris Yartsev
 */
public class InfraredReceiver implements Automaton, DrawableObject, CoordinateObject {
    private int state = IDLE_STATE;

    public static final int IDLE_STATE = 1;
    public static final int RECEIVING_STATE = 2;

    public static final int E1_ADD_MESSAGE_EVENT = 1;
    public static final int E2_REMOVE_MESSAGE_EVENT = 2;
    public static final int E3_DISABLE_EVENT = 3;
    public static final int E4_ENABLE_EVENT = 4;

    private double recieveAngle;

    private Queue messageQueue;

    private List autoUpdateList;

    /**
     * These are the coordinates of the reciever on the field, they are updated by the robot,
     * the reciever belongs to
     */
    private double x = 0;
    private double y = 0;
    private double angle = 0;

    private String receiverImageLocation = "images\\robots\\irdarec.GIF";
    private Image receiverImage = Toolkit.getDefaultToolkit().createImage(receiverImageLocation);

    private boolean shown = true;

    /**
     * This is the default constructor of the infrared reciever - it will be used in the emulating program
     */
    public InfraredReceiver() {
        this(2 * Math.PI);
    }

    /**
     * The recieving angle
     * @param angle
     */
    public InfraredReceiver(double angle) {
        messageQueue = new LinkedList();
        autoUpdateList = new ArrayList();
        recieveAngle = angle;
    }

    public double getRecieveAngle() {
        return recieveAngle;
    }

    public synchronized void updateAutoState(int event, int data) {
        switch(getAutoState()) {
            case RECEIVING_STATE:
                switch(event) {
                    case E1_ADD_MESSAGE_EVENT:
                        z0_addMessageToQueue((byte) data);
                        z2_updateAutomatonUpdators();
                        break;

                    case E2_REMOVE_MESSAGE_EVENT:
                        z1_removeMessageFromQueue();
                        break;

                    case E3_DISABLE_EVENT:
                        setAutoState(IDLE_STATE);
                        break;
                }
                break;

            case IDLE_STATE:
                switch(event) {
                    case E4_ENABLE_EVENT:
                        setAutoState(RECEIVING_STATE);
                        break;
                }
                break;
        }
    }

    private void z0_addMessageToQueue(byte value) {
        messageQueue.offer(new Byte(value));
    }

    private void z1_removeMessageFromQueue() {
        messageQueue.remove();
    }

    private void z2_updateAutomatonUpdators() {
        for (Iterator iter = autoUpdateList.iterator(); iter.hasNext(); ) {
            AutomatonUpdator autoU = (AutomatonUpdator) iter.next();
            autoU.updateAutomaton(0);
        }
    }

    public boolean hasMessage() {
        return messageQueue.size() > 0;
    }

    public void addMessage(byte message) {
        updateAutoState(E1_ADD_MESSAGE_EVENT, message);
    }

    public byte getMessage() {
        byte message = ((Byte) messageQueue.peek()).byteValue();
        updateAutoState(E2_REMOVE_MESSAGE_EVENT, 0);
        return message;
    }

    public int getAutoState() {
        return state;
    }

    public void setAutoState(int state) {
        this.state = state;
    }

    public void disable() {
        updateAutoState(E3_DISABLE_EVENT, 0);
    }

    public void enable() {
        updateAutoState(E4_ENABLE_EVENT, 0);
    }

    public double getX() {
        return x;
    }

    public void setX(double x) {
        this.x = x;
    }

    public double getY() {
        return y;
    }

    public void setY(double y) {
        this.y = y;
    }

    public double getAngle() {
        return angle;
    }

    public void setAngle(double angle) {
        this.angle = angle;
    }

    public Image getObjectImage() {
        return receiverImage;
    }

    public void setObjectImage(Image img) {
        receiverImage = img;
    }

    /*
    public Image getObjectImage() {
        return zoomer.getBufferImageOfFactor(zoomRatio);
    }
    */

    /*
    public void setObjectImage(Image img) {
        zoomer = new ImageZoomer(img);
    }
    */

    public boolean isShown() {
        return shown;
    }

    public void setShown(boolean shown) {
        this.shown = shown;
    }

    public synchronized void addAutomatonUpdatorOnReceiving(Automaton auto, int event) {
        AutomatonUpdator autoU = new AutomatonUpdator(auto, event);
        autoUpdateList.add(autoU);
    }

    /*
    public synchronized void removeAutomatonUpdatorOnReceiving(Automaton auto, int event) {
        System.out.println("Not supported");
        System.exit(1);
    }
    */

    public String toString() {
        return "Receiver: State = " + getAutoState() + " " + "Size = " + messageQueue.size() + " ; ";
    }
}