package robots.comm;

import robots.view.DrawableObject;
import robots.field.CoordinateObject;

import java.awt.*;

/**
 * In this class
 * @author Boris Yartsev
 */
public class InfraredReceiverEmitter implements DrawableObject, CoordinateObject {
    private InfraredReceiver receiver;
    private InfraredEmitter emitter;

    private String receiverEmitterLocation = "images\\robots\\irdaem.GIF";
    private Image receiverEmitterImage = Toolkit.getDefaultToolkit().createImage(receiverEmitterLocation);

    private boolean shown = true;

    private double delta = 0.1;

    public InfraredReceiverEmitter() {
        this(2 * Math.PI, Color.RED, 150, Math.PI / 3, 0.2);
    }

    public InfraredReceiverEmitter(double receiveAngle, Color color,
                                   double emittingRadius, double emittingAngle, double period) {
        receiver = new InfraredReceiver(receiveAngle);
        emitter = new InfraredEmitter(color, emittingRadius, emittingAngle, period);

        setAngle(0);
        setX(0);
        setY(0);

        Image img = Toolkit.getDefaultToolkit().createImage(receiverEmitterLocation);
        emitter.setObjectImage(img);

        receiver.setShown(false);
        emitter.setShown(true);
    }

    public void setX(double x) {
        double angle = receiver.getAngle() + Math.PI;
        emitter.setX(x);

        receiver.setX(emitter.getX() + delta * Math.cos(angle));
        receiver.setY(emitter.getY() + delta * Math.sin(angle));
    }

    public void setY(double y) {
        double angle = receiver.getAngle() + Math.PI;
        emitter.setY(y);

        receiver.setX(emitter.getX() + delta * Math.cos(angle));
        receiver.setY(emitter.getY() + delta * Math.sin(angle));
    }

    public void setAngle(double ang) {
        double angle = ang + Math.PI;

        receiver.setAngle(ang);
        emitter.setAngle(ang);

        receiver.setX(emitter.getX() + delta * Math.cos(angle));
        receiver.setY(emitter.getY() + delta * Math.sin(angle));
    }

    public double getX() {
        return emitter.getX();
    }

    public double getY() {
        return emitter.getY();
    }

    public double getAngle() {
        return emitter.getAngle();
    }

    public InfraredReceiver getReceiver() {
        return receiver;
    }

    public InfraredEmitter getEmitter() {
        return emitter;
    }

    public Image getObjectImage() {
        return receiverEmitterImage;
    }

    public void setObjectImage(Image img) {
        this.receiverEmitterImage = img;
    }

    /*
    public Image getObjectImage() {
        return zoomer.getBufferImageOfFactor(zoomRatio);
    }

    public void setObjectImage(Image img) {
        zoomer = new ImageZoomer(img);
    }
    */

    public boolean isShown() {
        return shown;
    }

    public void setShown(boolean shown) {
        this.shown = shown;

        if (shown) {
            emitter.setShown(true);
            receiver.setShown(false);
        } else {
            emitter.setShown(false);
            receiver.setShown(false);
        }
    }

    public String toString() {
        return "IRRECEM: " + receiver.toString() + emitter.toString();
    }
}