package robots.remote;

import java.awt.*;

import auto.Automaton;
import robots.view.DrawableObject;
import robots.comm.InfraredEmitter;
import robots.field.CoordinateObject;

/**
 * @author Boris Yartsev
 */
public class RemoteControl implements DrawableObject, Automaton, CoordinateObject {
    public static final int REMOTE_BUTTON1 = 1;
    public static final int REMOTE_BUTTON2 = 2;
    public static final int REMOTE_BUTTON3 = 3;

    private int state = BUTTON_PREPARED_STATE;

    public static final int BUTTON_PREPARED_STATE = 1;
    public static final int EMITTING_STATE = 2;

    public static final int E1_PRESSED_BUTTON_EVENT = 1;
    public static final int E2_PREPARE_BUTTON_EVENT = 2;

    private String remoteImageLocation = "images\\robots\\remote.gif";
    private Image remoteImage = Toolkit.getDefaultToolkit().createImage(remoteImageLocation);

    private String remote1ImageLocation = "images\\robots\\remote1.gif";
    private Image remote1Image = Toolkit.getDefaultToolkit().createImage(remote1ImageLocation);
    private Image remotePressed1Image = Toolkit.getDefaultToolkit().createImage("images\\robots\\remotep1.gif");

    private String remote2ImageLocation = "images\\robots\\remote2.gif";
    private Image remote2Image = Toolkit.getDefaultToolkit().createImage(remote2ImageLocation);
    private Image remotePressed2Image = Toolkit.getDefaultToolkit().createImage("images\\robots\\remotep2.gif");

    private String remote3ImageLocation = "images\\robots\\remote3.gif";
    private Image remote3Image = Toolkit.getDefaultToolkit().createImage(remote3ImageLocation);
    private Image remotePressed3Image = Toolkit.getDefaultToolkit().createImage("images\\robots\\remotep3.gif");

    private Image currentImage;

    private boolean shown = true;

    private double emRadius;
    private double emAngle;

    private InfraredEmitter emitter;

    private double x = 0;
    private double y = 0;
    private double angle = 0;

    private int pressedButton = -1;

    public RemoteControl() {
        this(15, 0);
    }

    public RemoteControl(double rad, double angle) {
        emitter = new InfraredEmitter();
        emitter.setShown(false);
        this.emRadius = rad;
        this.emAngle = angle;
        currentImage = remoteImage;
    }

    public double getX() {
        return x;
    }

    public void setX(double x) {
        this.x = x;
        setAngle(angle);
    }

    public double getY() {
        return y;
    }

    public void setY(double y) {
        this.y = y;
        setAngle(angle);
    }

    public double getAngle() {
        return angle;
    }

    public void setAngle(double angle) {
        this.angle = angle;
        emitter.setX(x + emRadius * Math.cos(angle + emAngle));
        emitter.setY(y + emRadius * Math.sin(angle + emAngle));
        emitter.setAngle(angle + emAngle);
    }

    public InfraredEmitter getEmitter() {
        return emitter;
    }

    public synchronized void updateAutoState(int event, int data) {
        switch(getAutoState()) {
            case BUTTON_PREPARED_STATE: {
                switch(event) {
                    case E1_PRESSED_BUTTON_EVENT:
                        z5_startSending();
                        z6_2_setSendingImage();
                        setAutoState(EMITTING_STATE);
                        break;

                    case E2_PREPARE_BUTTON_EVENT:
                        z3_storeSendValue(data);
                        z6_1_setPrepareImage();
                        break;
                }
                break;
            }

            case EMITTING_STATE: {
                switch(event) {
                    case E2_PREPARE_BUTTON_EVENT:
                        z3_storeSendValue(data);
                        z6_1_setPrepareImage();
                        z7_stopSending();
                        setAutoState(BUTTON_PREPARED_STATE);
                        break;

                    case E1_PRESSED_BUTTON_EVENT:
                        z6_1_setPrepareImage();
                        z7_stopSending();
                        setAutoState(BUTTON_PREPARED_STATE);
                        break;
                }
                break;
            }
        }
    }

    private void z3_storeSendValue(int value) {
        this.pressedButton = value;
    }

    private void z5_startSending() {
        emitter.startSending((byte) pressedButton);
    }

    private void z6_1_setPrepareImage() {
        switch(getPressedButton()) {
            case REMOTE_BUTTON1:
                currentImage = remote1Image;
                break;

            case REMOTE_BUTTON2:
                currentImage = remote2Image;
                break;

            case REMOTE_BUTTON3:
                currentImage = remote3Image;
                break;

            default:
                currentImage = remoteImage;
                break;
        }
    }

    private void z6_2_setSendingImage() {
        switch(getPressedButton()) {
            case REMOTE_BUTTON1:
                currentImage = remotePressed1Image;
                break;

            case REMOTE_BUTTON2:
                currentImage = remotePressed2Image;
                break;

            case REMOTE_BUTTON3:
                currentImage = remotePressed3Image;
                break;

            default:
                currentImage = remoteImage;
                break;
        }
    }

    private void z7_stopSending() {
        emitter.stopSending();
    }

    public void prepareButton(int button) {
        updateAutoState(E2_PREPARE_BUTTON_EVENT, button);
    }

    public void buttonPressed() {
        updateAutoState(E1_PRESSED_BUTTON_EVENT, getPressedButton());
    }

    public int getAutoState() {
        return state;
    }

    public void setAutoState(int state) {
        this.state = state;
    }

    private int getPressedButton() {
        return pressedButton;
    }

    public Image getObjectImage() {
        return currentImage;
    }

    public void setObjectImage(Image img) {
    }

    public boolean isShown() {
        return shown;
    }

    public void setShown(boolean shown) {
        this.shown = shown;
    }
}
