package robots.sensors;

import auto.Automaton;
import robots.view.DrawableObject;
import robots.view.ImageZoomer;
import robots.field.CoordinateObject;

import java.awt.*;

/**
 * @author Boris Yartsev
 */
public class LightSensor implements Automaton, Sensor, DrawableObject, CoordinateObject {
    private int state = NOT_ACTIVATED_STATE;

    public static final int NOT_ACTIVATED_STATE = 1;
    public static final int ACTIVATED_STATE = 2;

    //events
    public static final int E1_ACTIVATE_EVENT = -1;
    public static final int E2_DEACTIVATE_EVENT = 2;
    public static final int E3_LIGHT_CHANGED_EVENT = 1;

    private int intensivity = 0;

    private double x;
    private double y;
    private double angle;

    private boolean shown = true;

    private String lightSensorImageLocation = "images\\robots\\lightsensor.gif";
    private Image lightSensorImage = Toolkit.getDefaultToolkit().createImage(lightSensorImageLocation);

    private String description;

    public LightSensor(String description) {
        this.description = description;
    }

    public String getDescription() {
        return description;
    }

    public synchronized void updateAutoState(int event, int data) {
        switch(state) {
            case ACTIVATED_STATE:
                switch(event) {
                    case E3_LIGHT_CHANGED_EVENT:
                        z1_updateIntensivity(data);
                        break;
                    case E2_DEACTIVATE_EVENT:
                        setAutoState(NOT_ACTIVATED_STATE);
                        break;
                }
                break;

            case NOT_ACTIVATED_STATE:
                switch(event) {
                    case E1_ACTIVATE_EVENT:
                        setAutoState(ACTIVATED_STATE);
                        break;
                }
                break;
        }
    }


    private void z1_updateIntensivity(int value) {
        intensivity = value;
    }

    public void setX(double x) {
        this.x = x;
    }

    public void setY(double y) {
        this.y = y;
    }

    public void setAngle(double angle) {
        this.angle = angle;
    }

    public double getX() {
        return x;
    }

    public double getY() {
        return y;
    }

    public double getAngle() {
        return angle;
    }

    public int getAutoState() {
        return state;
    }

    public void setAutoState(int state) {
        this.state = state;
    }

    public int getValue() {
        return intensivity;
    }

    public void activate() {
        updateAutoState(E1_ACTIVATE_EVENT, 0);
    }

    public void deactivate() {
        updateAutoState(E2_DEACTIVATE_EVENT, 0);
    }

    public void intensityChanged(int intensity) {
        updateAutoState(E3_LIGHT_CHANGED_EVENT, intensity);
    }

    public boolean isShown() {
        return shown;
    }

    public void setShown(boolean shown) {
        this.shown = shown;
    }

    /*
    public Image getObjectImage() {
        return zoomer.getBufferImageOfFactor(getZoomRatio());
    }

    public void setObjectImage(Image img) {
        zoomer = new ImageZoomer(img);
    }
    */

    public Image getObjectImage() {
        return lightSensorImage;
    }

    public void setObjectImage(Image img) {
        this.lightSensorImage = img;
    }

    public String toString() {
        return "LightSensorValue: " + getValue();
    }
}
