package robots.sensors;

import auto.Automaton;

/**
 * @author Boris Yartsev
 */
public class TouchSensor implements Automaton, Sensor {
    public static final int NOT_TOUCHED_STATE = 1;
    public static final int TOUCHED_STATE = 2;

    public static final int E1_TOUCH_EVENT = 1;
    public static final int E2_UNTOUCH_EVENT = 2;

    private int state = NOT_TOUCHED_STATE;

    private String description;

    public TouchSensor(String description) {
        this.description = description;
    }

    public String getDescription() {
        return description;
    }

    public void touch() {
        updateAutoState(E1_TOUCH_EVENT, 0);
    }

    public void untouch() {
        updateAutoState(E2_UNTOUCH_EVENT, 0);
    }

    public int getValue() {
        return state;
    }

    public synchronized void updateAutoState(int event, int data) {
        switch(getAutoState()) {
            case TOUCHED_STATE:
                switch(event) {
                    case E2_UNTOUCH_EVENT:
                        setAutoState(NOT_TOUCHED_STATE);
                        break;

                }
                break;

            case NOT_TOUCHED_STATE:
                switch(event) {
                    case E1_TOUCH_EVENT:
                        setAutoState(TOUCHED_STATE);
                        break;
                }
                break;
        }
    }

    public int getAutoState() {
        return state;
    }

    public void setAutoState(int state) {
        this.state = state;
    }

    public String toString() {
        return "TouchSensorValue: " + state;
    }
}