package robots.transport;

import physics.PhysicalObject;

import java.awt.*;

/**
 * @author Boris Yartsev
 */
public class Rod extends PhysicalObject {
    public static final int TURN_LEFT_CALC_STATE = 1;
    public static final int TURN_RIGHT_CALC_STATE = 2;
    public static final int STOP_CALC_STATE = 3;

    public static final int E10_MOVE_LEFT_CALC_EVENT = 1;
    public static final int E11_MOVE_RIGHT_CALC_EVENT = 2;
    public static final int E12_STOP_CALC_EVENT = 3;
    public static final int E13_SET_POWER_CALC_EVENT = 4;
    public static final int E14_REVERSE_CALC_EVENT = 6;

    private double initialAngle;

    private double radius;

    private int power = 0;
    private double torquePerPowerUnit = 0.2;
    private double torqueAcc = 2;

    private boolean shown = true;

    public Rod(double rodRadius, double rodAngle) {
        super(0, 0, 0);
        this.radius = rodRadius;
        angle = initialAngle = rodAngle;
        setCalcState(STOP_CALC_STATE);
    }

    public int getPower() {
        return power;
    }

    public double getRadius() {
        return radius;
    }

    public double getInitialAngle() {
        return initialAngle;
    }

    public void setPower(int power) {
        updateCalcState(E13_SET_POWER_CALC_EVENT, power, 0, null);
    }

    public void turnLeft(int power) {
        updateCalcState(E10_MOVE_LEFT_CALC_EVENT, power, 0, null);
    }

    public void turnRight(int power) {
        updateCalcState(E11_MOVE_RIGHT_CALC_EVENT, power, 0, null);
    }

    public void stop() {
        updateCalcState(E12_STOP_CALC_EVENT, 0, 0, null);
    }

    public void reverse() {
        updateCalcState(E14_REVERSE_CALC_EVENT, 0, 0, null);
    }

    public void restart() {
        updateCalcState(E101_RESTART_CALC_EVENT, 0, 0, null);
    }

    public synchronized void updateCalcState(int event, int data, double delta, Object objData) {
        switch(getCalcState()) {
            case TURN_LEFT_CALC_STATE: {
                switch(event) {
                    case E10_MOVE_LEFT_CALC_EVENT:
                        z0_setPower(data);
                        break;

                    case E11_MOVE_RIGHT_CALC_EVENT:
                        z0_setPower(data);
                        setCalcState(TURN_RIGHT_CALC_STATE);
                        break;

                    case E12_STOP_CALC_EVENT:
                        setCalcState(STOP_CALC_STATE);
                        break;

                    case E13_SET_POWER_CALC_EVENT:
                        z0_setPower(data);
                        break;

                    case PhysicalObject.E101_RESTART_CALC_EVENT:
                        z1_restart();
                        setCalcState(STOP_CALC_STATE);
                        break;

                    case E14_REVERSE_CALC_EVENT:
                        setCalcState(TURN_RIGHT_CALC_STATE);
                        break;

                    case PhysicalObject.E100_UPDATE_COORDINATES_CALC_EVENT:
                        z3_1_updateCoordinatesLeft(delta);
                        z3_4_updateCoordinates(delta);
                        break;
                }
                break;
            }

            case TURN_RIGHT_CALC_STATE: {
                switch(event) {
                    case E10_MOVE_LEFT_CALC_EVENT:
                        z0_setPower(data);
                        setCalcState(TURN_LEFT_CALC_STATE);
                        break;

                    case E11_MOVE_RIGHT_CALC_EVENT:
                        z0_setPower(data);
                        break;

                    case E12_STOP_CALC_EVENT:
                        setCalcState(STOP_CALC_STATE);
                        break;

                    case E13_SET_POWER_CALC_EVENT:
                        z0_setPower(data);
                        break;

                    case PhysicalObject.E101_RESTART_CALC_EVENT:
                        z1_restart();
                        setCalcState(STOP_CALC_STATE);
                        break;

                    case E14_REVERSE_CALC_EVENT:
                        setCalcState(TURN_LEFT_CALC_STATE);
                        break;

                    case PhysicalObject.E100_UPDATE_COORDINATES_CALC_EVENT:
                        z3_2_updateCoordinatesRight(delta);
                        z3_4_updateCoordinates(delta);
                        break;
                }
                break;
            }

            case STOP_CALC_STATE: {
                switch(event) {
                    case E10_MOVE_LEFT_CALC_EVENT:
                        z0_setPower(data);
                        setCalcState(TURN_LEFT_CALC_STATE);
                        break;

                    case E11_MOVE_RIGHT_CALC_EVENT:
                        z0_setPower(data);
                        setCalcState(TURN_RIGHT_CALC_STATE);
                        break;

                    case E13_SET_POWER_CALC_EVENT:
                        z0_setPower(data);
                        break;

                    case PhysicalObject.E101_RESTART_CALC_EVENT:
                        z1_restart();
                        break;

                    case PhysicalObject.E100_UPDATE_COORDINATES_CALC_EVENT:
                        z3_3_updateCoordinatesStop(delta);
                        z3_4_updateCoordinates(delta);
                        break;
                }
                break;
            }


        }
    }

    private void z0_setPower(int power) {
        this.power = power;
    }

    private void z1_restart() {
        power = 0;
        angle = initialAngle;
        wTorque = 0;
    }

    /*
    private void z3_updateCoordinates(double timeDelta) {
        switch(getCalcState()) {
            case TURN_LEFT_CALC_STATE: {

                if (wTorque < torquePerPowerUnit * power) {
                    wTorque += torqueAcc * timeDelta;
                } else {
                    wTorque -= torqueAcc * timeDelta;
                }

                break;
            }

            case TURN_RIGHT_CALC_STATE: {
                if (wTorque > -torquePerPowerUnit * power) {
                    wTorque -= torqueAcc * timeDelta;
                } else {
                    wTorque += torqueAcc * timeDelta;
                }

                break;
            }

            case STOP_CALC_STATE: {

                if (Math.abs(wTorque) > 0) {
                    double signum = Math.signum(wTorque);
                    wTorque += -signum * torqueAcc * timeDelta;
                    double newSignum = Math.signum(wTorque);

                    if (newSignum * signum < 0) {
                        wTorque = 0;
                    }
                }

                break;
            }
        }

        angle += wTorque * timeDelta;
    }
    */

    private void z3_1_updateCoordinatesLeft(double timeDelta) {
        if (wTorque < torquePerPowerUnit * power) {
            wTorque += torqueAcc * timeDelta;
        } else {
            wTorque -= torqueAcc * timeDelta;
        }
    }

    private void z3_2_updateCoordinatesRight(double timeDelta) {
        if (wTorque > -torquePerPowerUnit * power) {
            wTorque -= torqueAcc * timeDelta;
        } else {
            wTorque += torqueAcc * timeDelta;
        }
    }

    private void z3_3_updateCoordinatesStop(double timeDelta) {
        if (Math.abs(wTorque) > 0) {
            double signum = Math.signum(wTorque);
            wTorque += -signum * torqueAcc * timeDelta;
            double newSignum = Math.signum(wTorque);

            if (newSignum * signum < 0) {
                wTorque = 0;
            }
        }
    }

    private void z3_4_updateCoordinates(double timeDelta) {
        angle += wTorque * timeDelta;
    }

    public Image getObjectImage() {
        return null;
    }

    public void setObjectImage(Image img) {
        System.out.println("Images for the ROD are still not supported");
    }

    public boolean isShown() {
        return shown;
    }

    public void setShown(boolean shown) {
        this.shown = shown;
    }

}
