package robots.view;

import auto.Automaton;

import java.awt.*;
import java.awt.image.BufferedImage;

import physics.Utilities;

/**
 * @author Boris Yartsev
 */
public class BattleFieldEditor implements Automaton {
    public static final int DRAW_LINE_STATE = 1;
    public static final int PLACE_ROBOT_STATE = 2;
    public static final int ROTATE_ROBOT_STATE = 3;
    public static final int ROBOT_PLACED_STATE = 4;
    public static final int ROBOT_ROTATED_STATE = 5;

    private int state = DRAW_LINE_STATE;

    public static final int TRANSPORT_ROBOT = 1;
    public static final int DISPENCER_ROBOT = 2;

    private int robotType;

    public static final int ROUND_BRUSH = 1;
    public static final int SQUARE_BRUSH = 2;

    private Image viewImage;
    private Image buffer;
    private Graphics viewImageGraphics;

    private int prevX;
    private int prevY;

    private int currobX = 0;
    private int currobY = 0;
    private double currobAngle = 0;

    private int trX = 0;
    private int trY = 0;
    private double trAngle = 0;

    private int disX = 0;
    private int disY = 0;
    private double disAngle = 0;
    private Color fieldColor;

    private Image pathImageBrush;
    private Image fieldImageBrush;
    private Image foilImageBrush;
    private Image errorImageBrush;

    private Image activeBrush;

    public BattleFieldEditor(Image pi, Image fi, Image foiI, Image erI, Color fieldColor) {
        pathImageBrush = pi;
        fieldImageBrush = fi;
        foilImageBrush = foiI;
        errorImageBrush = erI;
        activeBrush = pathImageBrush;
        this.fieldColor = fieldColor;
    }

    public void loadNewImage(Image offscreen) {
        viewImage = offscreen;
        viewImageGraphics = viewImage.getGraphics();
        buffer = new BufferedImage(viewImage.getWidth(null), viewImage.getHeight(null), BufferedImage.TYPE_INT_ARGB);
        buffer.getGraphics().drawImage(viewImage, 0, 0, null);
    }

    public synchronized void updateAutoState(int event, int data) {
        switch(state) {
            case DRAW_LINE_STATE: {
                switch(event) {
                    case BattleFieldView.E17_LEFT_MOUSE_DRAGGED_EVENT: {
                        z1_drawLine((data >> 16) & 0xffff, data & 0xffff);
                        z2_saveCoords((data >> 16) & 0xffff, data & 0xffff);
                        break;
                    }

                    case BattleFieldView.E19_LEFT_MOUSE_PRESSED_EVENT: {
                        z2_saveCoords((data >> 16) & 0xffff, data & 0xffff);
                        z1_drawLine((data >> 16) & 0xffff, data & 0xffff);
                        break;
                    }

                    case BattleFieldView.E21_PLACE_TRANSPORT_BUTTON_PRESSED_EVENT: {
                        z0_1_placeTransport();
                        setAutoState(PLACE_ROBOT_STATE);
                        break;
                    }

                    case BattleFieldView.E22_PLACE_DISPENCER_BUTTON_PRESSED_EVENT: {
                        z0_2_placeDispencer();
                        setAutoState(PLACE_ROBOT_STATE);
                        break;
                    }

                    case BattleFieldView.E23_ROTATE_TRANSPORT_BUTTON_PRESSED_EVENT: {
                        z0_1_placeTransport();
                        setAutoState(ROTATE_ROBOT_STATE);
                        break;
                    }

                    case BattleFieldView.E24_ROTATE_DISPENCER_BUTTON_PRESSED_EVENT: {
                        z0_2_placeDispencer();
                        setAutoState(ROTATE_ROBOT_STATE);
                        break;
                    }

                    case BattleFieldView.E25_DRAW_PATH_BUTTON_PRESSED_EVENT: {
                        z10_preparePath();
                        break;
                    }

                    case BattleFieldView.E26_DRAW_FOIL_BUTTON_PRESSED_EVENT: {
                        z7_prepareFoil();
                        break;
                    }

                    case BattleFieldView.E27_DRAW_ERROR_BUTTON_PRESSED_EVENT: {
                        z8_prepareError();
                        break;
                    }

                    case BattleFieldView.E28_CLEAR_BUTTON_PRESSED_EVENT: {
                        z9_prepareClear();
                        break;
                    }

                    case BattleFieldView.E29_CLEAR_ALL_BUTTON_PRESSED_EVENT: {
                        z14_flushImage();
                        break;
                    }

                    case BattleFieldView.E30_OK_BUTTON_PRESSED_EVENT: {
                        z13_storeImage();
                        z10_preparePath();
                        break;
                    }

                    case BattleFieldView.E31_CANCEL_BUTTON_PRESSED_EVENT: {
                        z12_restoreImage();
                        z10_preparePath();
                        break;
                    }
                }
                break;
            }

            case PLACE_ROBOT_STATE: {
                switch(event) {
                    case BattleFieldView.E16_MOUSE_MOVED_EVENT: {
                        z3_updateRobotCoords((data >> 16) & 0xffff, data & 0xffff);
                        break;
                    }

                    case BattleFieldView.E20_LEFT_MOUSE_CLICKED_EVENT: {
                        z3_updateRobotCoords((data >> 16) & 0xffff, data & 0xffff);
                        z4_storeRobotCoords();
                        setAutoState(ROBOT_PLACED_STATE);
                        break;
                    }

                    case BattleFieldView.E21_PLACE_TRANSPORT_BUTTON_PRESSED_EVENT: {
                        z0_1_placeTransport();
                        break;
                    }

                    case BattleFieldView.E22_PLACE_DISPENCER_BUTTON_PRESSED_EVENT: {
                        z0_2_placeDispencer();
                        break;
                    }

                    case BattleFieldView.E23_ROTATE_TRANSPORT_BUTTON_PRESSED_EVENT: {
                        z0_1_placeTransport();
                        setAutoState(ROTATE_ROBOT_STATE);
                        break;
                    }

                    case BattleFieldView.E24_ROTATE_DISPENCER_BUTTON_PRESSED_EVENT: {
                        z0_2_placeDispencer();
                        setAutoState(ROTATE_ROBOT_STATE);
                        break;
                    }

                    case BattleFieldView.E25_DRAW_PATH_BUTTON_PRESSED_EVENT: {
                        z10_preparePath();
                        setAutoState(DRAW_LINE_STATE);
                        break;
                    }

                    case BattleFieldView.E26_DRAW_FOIL_BUTTON_PRESSED_EVENT: {
                        z7_prepareFoil();
                        setAutoState(DRAW_LINE_STATE);
                        break;
                    }

                    case BattleFieldView.E27_DRAW_ERROR_BUTTON_PRESSED_EVENT: {
                        z8_prepareError();
                        setAutoState(DRAW_LINE_STATE);
                        break;
                    }

                    case BattleFieldView.E28_CLEAR_BUTTON_PRESSED_EVENT: {
                        z9_prepareClear();
                        setAutoState(DRAW_LINE_STATE);
                        break;
                    }

                    case BattleFieldView.E29_CLEAR_ALL_BUTTON_PRESSED_EVENT: {
                        z14_flushImage();
                        break;
                    }

                    case BattleFieldView.E30_OK_BUTTON_PRESSED_EVENT: {
                        z13_storeImage();
                        z10_preparePath();
                        setAutoState(DRAW_LINE_STATE);
                        break;
                    }

                    case BattleFieldView.E31_CANCEL_BUTTON_PRESSED_EVENT: {
                        z12_restoreImage();
                        z10_preparePath();
                        setAutoState(DRAW_LINE_STATE);
                        break;
                    }
                }
                break;
            }

            case ROTATE_ROBOT_STATE: {
                switch(event) {
                    case BattleFieldView.E16_MOUSE_MOVED_EVENT: {
                        z5_updateRobotAngle((data >> 16) & 0xffff, data & 0xffff);
                        break;
                    }

                    case BattleFieldView.E20_LEFT_MOUSE_CLICKED_EVENT: {
                        z6_storeRobotAngle();
                        setAutoState(ROBOT_ROTATED_STATE);
                        break;
                    }

                    case BattleFieldView.E21_PLACE_TRANSPORT_BUTTON_PRESSED_EVENT: {
                        z0_1_placeTransport();
                        setAutoState(PLACE_ROBOT_STATE);
                        break;
                    }

                    case BattleFieldView.E22_PLACE_DISPENCER_BUTTON_PRESSED_EVENT: {
                        z0_2_placeDispencer();
                        setAutoState(PLACE_ROBOT_STATE);
                        break;
                    }

                    case BattleFieldView.E23_ROTATE_TRANSPORT_BUTTON_PRESSED_EVENT: {
                        z0_1_placeTransport();
                        break;
                    }

                    case BattleFieldView.E24_ROTATE_DISPENCER_BUTTON_PRESSED_EVENT: {
                        z0_2_placeDispencer();
                        break;
                    }

                    case BattleFieldView.E25_DRAW_PATH_BUTTON_PRESSED_EVENT: {
                        z10_preparePath();
                        setAutoState(DRAW_LINE_STATE);
                        break;
                    }

                    case BattleFieldView.E26_DRAW_FOIL_BUTTON_PRESSED_EVENT: {
                        z7_prepareFoil();
                        setAutoState(DRAW_LINE_STATE);
                        break;
                    }

                    case BattleFieldView.E27_DRAW_ERROR_BUTTON_PRESSED_EVENT: {
                        z8_prepareError();
                        setAutoState(DRAW_LINE_STATE);
                        break;
                    }

                    case BattleFieldView.E28_CLEAR_BUTTON_PRESSED_EVENT: {
                        z9_prepareClear();
                        setAutoState(DRAW_LINE_STATE);
                        break;
                    }

                    case BattleFieldView.E29_CLEAR_ALL_BUTTON_PRESSED_EVENT: {
                        z14_flushImage();
                        break;
                    }

                    case BattleFieldView.E30_OK_BUTTON_PRESSED_EVENT: {
                        z13_storeImage();
                        z10_preparePath();
                        setAutoState(DRAW_LINE_STATE);
                        break;
                    }

                    case BattleFieldView.E31_CANCEL_BUTTON_PRESSED_EVENT: {
                        z12_restoreImage();
                        z10_preparePath();
                        setAutoState(DRAW_LINE_STATE);
                        break;
                    }
                }
                break;
            }

            case ROBOT_PLACED_STATE: {
                switch(event) {
                    case BattleFieldView.E20_LEFT_MOUSE_CLICKED_EVENT: {
                        z3_updateRobotCoords((data >> 16) & 0xffff, data & 0xffff);
                        setAutoState(PLACE_ROBOT_STATE);
                        break;
                    }

                    case BattleFieldView.E21_PLACE_TRANSPORT_BUTTON_PRESSED_EVENT: {
                        z0_1_placeTransport();
                        setAutoState(PLACE_ROBOT_STATE);
                        break;
                    }

                    case BattleFieldView.E22_PLACE_DISPENCER_BUTTON_PRESSED_EVENT: {
                        z0_2_placeDispencer();
                        setAutoState(PLACE_ROBOT_STATE);
                        break;
                    }

                    case BattleFieldView.E23_ROTATE_TRANSPORT_BUTTON_PRESSED_EVENT: {
                        z0_1_placeTransport();
                        setAutoState(ROTATE_ROBOT_STATE);
                        break;
                    }

                    case BattleFieldView.E24_ROTATE_DISPENCER_BUTTON_PRESSED_EVENT: {
                        z0_2_placeDispencer();
                        setAutoState(ROTATE_ROBOT_STATE);
                        break;
                    }

                    case BattleFieldView.E25_DRAW_PATH_BUTTON_PRESSED_EVENT: {
                        z10_preparePath();
                        setAutoState(DRAW_LINE_STATE);
                        break;
                    }

                    case BattleFieldView.E26_DRAW_FOIL_BUTTON_PRESSED_EVENT: {
                        z7_prepareFoil();
                        setAutoState(DRAW_LINE_STATE);
                        break;
                    }

                    case BattleFieldView.E27_DRAW_ERROR_BUTTON_PRESSED_EVENT: {
                        z8_prepareError();
                        setAutoState(DRAW_LINE_STATE);
                        break;
                    }

                    case BattleFieldView.E28_CLEAR_BUTTON_PRESSED_EVENT: {
                        z9_prepareClear();
                        setAutoState(DRAW_LINE_STATE);
                        break;
                    }

                    case BattleFieldView.E29_CLEAR_ALL_BUTTON_PRESSED_EVENT: {
                        z14_flushImage();
                        break;
                    }

                    case BattleFieldView.E30_OK_BUTTON_PRESSED_EVENT: {
                        z13_storeImage();
                        z10_preparePath();
                        setAutoState(DRAW_LINE_STATE);
                        break;
                    }

                    case BattleFieldView.E31_CANCEL_BUTTON_PRESSED_EVENT: {
                        z12_restoreImage();
                        z10_preparePath();
                        setAutoState(DRAW_LINE_STATE);
                        break;
                    }
                }
                break;
            }

            case ROBOT_ROTATED_STATE: {
                switch(event) {
                    case BattleFieldView.E20_LEFT_MOUSE_CLICKED_EVENT: {
                        z5_updateRobotAngle((data >> 16) & 0xffff, data & 0xffff);
                        setAutoState(ROTATE_ROBOT_STATE);
                        break;
                    }

                    case BattleFieldView.E21_PLACE_TRANSPORT_BUTTON_PRESSED_EVENT: {
                        z0_1_placeTransport();
                        setAutoState(PLACE_ROBOT_STATE);
                        break;
                    }

                    case BattleFieldView.E22_PLACE_DISPENCER_BUTTON_PRESSED_EVENT: {
                        z0_2_placeDispencer();
                        setAutoState(PLACE_ROBOT_STATE);
                        break;
                    }

                    case BattleFieldView.E23_ROTATE_TRANSPORT_BUTTON_PRESSED_EVENT: {
                        z0_1_placeTransport();
                        setAutoState(ROTATE_ROBOT_STATE);
                        break;
                    }

                    case BattleFieldView.E24_ROTATE_DISPENCER_BUTTON_PRESSED_EVENT: {
                        z0_2_placeDispencer();
                        setAutoState(ROTATE_ROBOT_STATE);
                        break;
                    }

                    case BattleFieldView.E25_DRAW_PATH_BUTTON_PRESSED_EVENT: {
                        z10_preparePath();
                        setAutoState(DRAW_LINE_STATE);
                        break;
                    }

                    case BattleFieldView.E26_DRAW_FOIL_BUTTON_PRESSED_EVENT: {
                        z7_prepareFoil();
                        setAutoState(DRAW_LINE_STATE);
                        break;
                    }

                    case BattleFieldView.E27_DRAW_ERROR_BUTTON_PRESSED_EVENT: {
                        z8_prepareError();
                        setAutoState(DRAW_LINE_STATE);
                        break;
                    }

                    case BattleFieldView.E28_CLEAR_BUTTON_PRESSED_EVENT: {
                        z9_prepareClear();
                        setAutoState(DRAW_LINE_STATE);
                        break;
                    }

                    case BattleFieldView.E29_CLEAR_ALL_BUTTON_PRESSED_EVENT: {
                        z14_flushImage();
                        break;
                    }

                    case BattleFieldView.E30_OK_BUTTON_PRESSED_EVENT: {
                        z13_storeImage();
                        z10_preparePath();
                        setAutoState(DRAW_LINE_STATE);
                        break;
                    }

                    case BattleFieldView.E31_CANCEL_BUTTON_PRESSED_EVENT: {
                        z12_restoreImage();
                        z10_preparePath();
                        setAutoState(DRAW_LINE_STATE);
                        break;
                    }
                }
                break;
            }
        }
    }

    public int getAutoState() {
        return state;
    }

    public void setAutoState(int state) {
        this.state = state;
    }

    public int getRobotX(int robot) {
        if ((getAutoState() == PLACE_ROBOT_STATE) && (robot == robotType)) {
            return currobX;
        } else {
            switch(robot) {
                case TRANSPORT_ROBOT:
                    return trX;

                case DISPENCER_ROBOT:
                    return disX;
            }
        }

        return 0;
    }

    public int getRobotY(int robot) {
        if ((getAutoState() == PLACE_ROBOT_STATE)  && (robot == robotType)) {
            return currobY;
        } else {
            switch(robot) {
                case TRANSPORT_ROBOT:
                    return trY;

                case DISPENCER_ROBOT:
                    return disY;
            }
        }

        return 0;
    }

    public double getRobotAngle(int robot) {
        if (getAutoState() == ROTATE_ROBOT_STATE && robot == robotType) {
            return currobAngle;
        } else {
            switch(robot) {
                case TRANSPORT_ROBOT:
                    return trAngle;

                case DISPENCER_ROBOT:
                    return disAngle;
            }
        }

        return 0;
    }


    public void setRobotCoords(int robot, int x, int y, double angle) {
        switch(robot) {
            case TRANSPORT_ROBOT:
                currobX = trX = x;
                currobY = trY = y;
                currobAngle = trAngle = angle;
                break;

            case DISPENCER_ROBOT:
                currobX = disX = x;
                currobY = disY = y;
                currobAngle = disAngle = angle;
                break;
        }
    }

    private void drawLine(int x, int y, Graphics g) {
        if ((x < 0) || (y < 0) || (x >= viewImage.getWidth(null)) || (y >= viewImage.getHeight(null))) {
            return;
        }

        int x1 = x;
        int x2 = prevX;
        int y1 = y;
        int y2 = prevY;
        int dx = ((x2 - x1) > 0) ? 1 : -1;
        int dy = ((y2 - y1) > 0) ? 1 : -1;
        int u = Math.abs(x2 - x1);
        int v = Math.abs(y2 - y1);

        if (v >= u) {
            int d = 2 * u - v;

            for (int i = 0; i <= v; i++) {
                g.drawImage(activeBrush, x1 - activeBrush.getWidth(null) / 2 + 1, y1 - activeBrush.getHeight(null) / 2 + 1, null);

                if (d < 0) {
                    y1 += dy;
                    d += 2 * u;
                } else {
                    x1 += dx;
                    y1 += dy;
                    d += 2 * u - 2 * v;
                }
            }
        } else {
            int d = 2 * v - u;

            for (int i = 0; i <= u; i++) {
                g.drawImage(activeBrush, x1 - activeBrush.getWidth(null) / 2 + 1, y1- activeBrush.getHeight(null) / 2 + 1, null);

                if (d < 0) {
                    x1 += dx;
                    d += 2 * v;
                } else {
                    x1 += dx;
                    y1 += dy;
                    d += 2 * v - 2 * u;
                }
            }
        }
    }

    /*
    //todo make normal rasteriation
    //todo make something clever with the brush
    private void myEllipse(int x, int y, Graphics g) {
        for (int i = -3; i <= 4; i++) {
            if ((i == -3) || (i == 4)) {
                g.drawLine(x + i, y - 2, x + i, y + 3);
            } else {
                g.drawLine(x + i, y - 3, x + i, y + 4);
            }
        }
    }

    private void mySquare(int x, int y, Graphics g) {
        for (int i = -4; i <= 5; i++) {
          g.drawLine(x + i, y - 4, x + i, y + 5);
        }
    }
    */

    private void z0_1_placeTransport() {
        robotType = TRANSPORT_ROBOT;
        currobX = trX;
        currobY = trY;
    }

    private void z0_2_placeDispencer() {
        robotType = DISPENCER_ROBOT;
        currobX = disX;
        currobY = disY;
    }

    private void z1_drawLine(int x, int y) {
        drawLine(x, y, viewImageGraphics);
    }


    private void z2_saveCoords(int x, int y) {
        if ((x < 0) || (y < 0) || (x >= viewImage.getWidth(null)) || (y >= viewImage.getHeight(null))) {
            return;
        }
        prevX = x;
        prevY = y;
    }

    private void z3_updateRobotCoords(int tx, int ty) {
        if ((tx >= 0) && (ty >= 0) &&
                (tx < viewImage.getWidth(null) && (ty < viewImage.getHeight(null)))) {
            currobX = tx;
            currobY = ty;
        }
    }

    private void z4_storeRobotCoords() {
        switch(robotType) {
            case TRANSPORT_ROBOT:
                trX = currobX;
                trY = currobY;
                break;

            case DISPENCER_ROBOT:
                disX = currobX;
                disY = currobY;
                break;
        }
    }

    private void z5_updateRobotAngle(int x, int y) {
        int tx = x - currobX;
        int ty = -y + currobY;
        currobAngle = Utilities.convertToAngle(tx, ty);
    }

    private void z6_storeRobotAngle() {
        switch(robotType) {
            case TRANSPORT_ROBOT:
                trAngle = currobAngle;
                break;

            case DISPENCER_ROBOT:
                disAngle = currobAngle;
                break;
        }
    }

    //------------------------------------------------------------------------------------------------------------------
    private void z7_prepareFoil() {
        activeBrush = foilImageBrush;
    }

    private void z8_prepareError() {
        activeBrush = errorImageBrush;
    }

    private void z9_prepareClear() {
        activeBrush = fieldImageBrush;
    }

    private void z10_preparePath() {
        activeBrush = pathImageBrush;
    }

    private void z12_restoreImage() {
        viewImageGraphics.drawImage(buffer, 0, 0, null);
    }

    private void z13_storeImage() {
        buffer.getGraphics().drawImage(viewImage, 0, 0, null);
    }

    private void z14_flushImage() {
        viewImageGraphics.setColor(fieldColor);
        viewImageGraphics.fillRect(0, 0, viewImage.getWidth(null), viewImage.getHeight(null));
    }
    //------------------------------------------------------------------------------------------------------------------
}