package robots.view;

import java.awt.*;

/**
 * @author Boris Yartsev
 */

public class CursorImageFactory {
    private static Image pathImage = Toolkit.getDefaultToolkit().createImage("images\\cursor\\pathImage.gif");
    private static Image foilImage = Toolkit.getDefaultToolkit().createImage("images\\cursor\\foilImage.gif");
    private static Image errorImage = Toolkit.getDefaultToolkit().createImage("images\\cursor\\errorImage.gif");
    private static Image clearImage = Toolkit.getDefaultToolkit().createImage("images\\cursor\\clearImage.gif");

    public static final int PATH_CURSOR_IMAGE = 1;
    public static final int FOIL_CURSOR_IMAGE = 2;
    public static final int ERROR_CURSOR_IMAGE = 3;
    public static final int CLEAR_CURSOR_IMAGE = 4;

    private static Image pathBrush = Toolkit.getDefaultToolkit().createImage("images\\brushes\\pathbrush.GIF");
    private static Image foilBrush = Toolkit.getDefaultToolkit().createImage("images\\brushes\\foilbrush.GIF");
    private static Image errorBrush = Toolkit.getDefaultToolkit().createImage("images\\brushes\\errorbrush.GIF");
    private static Image clearBrush = Toolkit.getDefaultToolkit().createImage("images\\brushes\\clearbrush.GIF");

    public static final int PATH_BRUSH = 5;
    public static final int FIELD_BRUSH = 6;
    public static final int FOIL_BRUSH = 7;
    public static final int ERROR_BRUSH = 8;

    public static class Brush {
        public Image brushImage;
        public Color color;

        public Brush(Image img, Color col) {
            brushImage = img;
            color = col;
        }

        public Image getBrushImage() {
            return brushImage;
        }

        public Color getColor() {
            return color;
        }
    }

    public static Image createImage(int type) {
        switch(type) {
            case PATH_CURSOR_IMAGE:
                return pathImage;

            case FOIL_CURSOR_IMAGE:
                return foilImage;

            case ERROR_CURSOR_IMAGE:
                return errorImage;

            case CLEAR_CURSOR_IMAGE:
                return clearImage;
        }

        return null;
    }


    public static Brush createBrush(int type) {
        switch(type) {
            case PATH_BRUSH:
                return new Brush(pathBrush, Color.BLACK);

            case FOIL_BRUSH:
                return new Brush(foilBrush, Color.GRAY);

            case ERROR_BRUSH:
                return new Brush(errorBrush, Color.RED);

            case FIELD_BRUSH:
                return new Brush(clearBrush, Color.WHITE);
        }

        return null;
    }

/*
    public static Image createImage(int type) {
        BufferedImage image = new BufferedImage(32, 32, BufferedImage.TYPE_INT_ARGB);
        WritableRaster raster = image.getRaster();
        int[] array = new int [4];

        array[0] = 0;
        array[1] = 0;
        array[2] = 0;
        array[3] = 0;

        for (int i = 0; i < 32; i++) {
            for (int j = 0; j < 32; j++) {
                raster.setPixel(i, j, array);
            }
        }

        if (type == MANDM_BLUE) {
            array[0] = 255;
            array[1] = 255;
            array[2] = 255;
            array[3] = 255;

            int x = 15;
            int y = 15;

            for (int i = -4; i <= 5; i++) {
                for (int j = -4; j <= 5; j++) {
                    raster.setPixel(x + i, y + j, array);
                }
            }

            array[0] = 0;
            array[1] = 0;
            array[2] = 0;
            array[3] = 255;

            for (int i = -4; i <= 5; i++) {
                raster.setPixel(x + i, y - 4, array);
                raster.setPixel(x + i, y + 5, array);
            }

            for (int i = -4; i <= 5; i++) {
                raster.setPixel(x - 4, y + i, array);
                raster.setPixel(x + 5, y + i, array);
            }
        } else

        if ((type == MANDM_GREEN) || (type == MANDM_YELLOW) || (type == MANDM_RED)) {

            int x = 15;
            int y = 15;

            switch(type) {
                case MANDM_GREEN:
                    array[0] = 0;
                    array[1] = 0;
                    array[2] = 0;
                    array[3] = 255;
                    break;

                case MANDM_YELLOW:
                    array[0] = 255;
                    array[1] = 0;
                    array[2] = 0;
                    array[3] = 255;
                    break;

                case MANDM_RED:
                    array[0] = 128;
                    array[1] = 128;
                    array[2] = 128;
                    array[3] = 255;
                    break;
            }

            for (int i = -3; i <= 4; i++) {
                for (int j = -3; j <= 4; j++) {
                    raster.setPixel(x + i, y + j, array);
                }
            }

            array[0] = 0;
            array[1] = 0;
            array[2] = 0;
            array[3] = 0;

            raster.setPixel(x - 3, y - 3, array);
            raster.setPixel(x - 3, y + 4, array);
            raster.setPixel(x + 4, y + 4, array);
            raster.setPixel(x + 4, y - 3, array);
        }
        return image;
    }
    */
}
