package robots.view;

import java.awt.*;
import java.awt.image.BufferedImage;

/**
 * @author Boris Yartsev
 */
public class ImageZoomer {

    //This is the default values for the zooming
    public static final int defaultZoomRatio[] = {
        1, 2, 3
    };

    private Image sourceImage = null;
    private Image images[];
    private Image bufferedImages[];
    private int factors[];
    private boolean loaded[];
    private boolean customImage[];

    /**
     * Creates the <code>ImageZoomer</code> object
     * @param zoomImage
     */
    public ImageZoomer(Image zoomImage) {
        this(zoomImage, defaultZoomRatio);
    }

    public ImageZoomer(Image zoomImages[]) {
        this(zoomImages, defaultZoomRatio);
    }

    private ImageZoomer(int zoomFactors[]) {
        images = new Image[zoomFactors.length];
        bufferedImages = new Image[zoomFactors.length];
        factors = new int[zoomFactors.length];
        loaded = new boolean[zoomFactors.length];
        customImage = new boolean[zoomFactors.length];

        for (int i = 0; i < zoomFactors.length; i++) {
            factors[i] = zoomFactors[i];
            loaded[i] = false;
            customImage[i] = false;
        }
    }

    public ImageZoomer(Image zoomImage, int zoomFactors[]) {
        this(zoomFactors);
        setSourceImage(zoomImage);
    }

    /**
     * It is very important, that the zoomImages.length == zoomFactors.length. No error is generated, but it
     * is unsecure
     * @param zoomImages
     * @param zoomFactors
     */
    public ImageZoomer(Image zoomImages[], int zoomFactors[]) {
        this(zoomFactors);

        for (int i = 0; i < zoomFactors.length; i++) {
            setCustomImage(zoomFactors[i], zoomImages[i]);
        }
    }

    private void initImage(int imageId) {
        if (customImage[imageId]) {
            if (!loaded[imageId]) {
                int w = images[imageId].getWidth(null);
                int h = images[imageId].getHeight(null);

                if (w > 0 && h > 0) {
                    bufferedImages[imageId] = new BufferedImage(w, h, BufferedImage.TYPE_INT_ARGB);
                    loaded[imageId] = true;
                }
            }
        } else {
            if (!loaded[imageId]) {
                int w = sourceImage.getWidth(null);
                int h = sourceImage.getHeight(null);

                if (w > 0 && h > 0) {
                    if (bufferedImages[imageId] == null) {
                        bufferedImages[imageId] =
                                new BufferedImage(w * factors[imageId], h * factors[imageId], BufferedImage.TYPE_INT_ARGB);
                    }
                    images[imageId] =
                            sourceImage.getScaledInstance(w * factors[imageId], h * factors[imageId], Image.SCALE_SMOOTH);
                    loaded[imageId] = true;
                }
            }
        }
    }


    /**
     * This method returns the prepared buffered image for drawing
     * @param factor
     * @return
     */
    public synchronized Image getBufferImageOfFactor(int factor) {
        for (int i = 0; i < factors.length; i++) {
            if (factor == factors[i]) {
                if (!loaded[i]) {
                    initImage(i);
                }

                if (loaded[i]) {
                    if (images[i].getWidth(null) > 0 && images[i].getHeight(null) > 0) {
                        bufferedImages[i].getGraphics().drawImage(images[i], 0, 0, null);
                    }
                }

                return bufferedImages[i];
            }
        }
        return null;
    }

    public void setCustomImage(int factor, Image customImg) {
        for (int i = 0; i < factors.length; i++) {
            if (factor == factors[i]) {
                images[i] = customImg;
                customImage[i] = true;
                int w = customImg.getWidth(null);
                int h = customImg.getHeight(null);

                if (w > 0 && h > 0) {
                    bufferedImages[i] = new BufferedImage(w, h, BufferedImage.TYPE_INT_ARGB);
                    loaded[i] = true;
                } else {
                    loaded[i] = false;
                }
            }
        }
    }

    /**
     * It is assumed, that the source image
     * @param sourceImage
     */
    public void setSourceImage(Image sourceImage) {
        this.sourceImage = sourceImage;

        int w = sourceImage.getWidth(null);
        int h = sourceImage.getHeight(null);

        for (int i = 0; i < factors.length; i++) {
            if (!customImage[i]) {
                if (w > 0 && h > 0) {
                    images[i] = sourceImage.getScaledInstance(w * factors[i], h * factors[i], Image.SCALE_SMOOTH);

                    if (bufferedImages[i] == null) {
                        bufferedImages[i] = new BufferedImage(w * factors[i], h * factors[i], BufferedImage.TYPE_INT_ARGB);
                    }

                    factors[i] = factors[i];
                    loaded[i] = true;
                    customImage[i] = false;
                } else {
                    factors[i] = factors[i];
                    loaded[i] = false;
                    customImage[i] = false;
                }
            }
        }
    }
}