package robots.view;

import robots.sensors.Sensor;
import robots.FieldRobot;

import javax.swing.*;
import java.util.Iterator;
import java.util.ArrayList;
import java.awt.*;

import auto.ControlAutomaton;

/**
 * @author Boris Yartsev
 */
public class RobotDataOutput extends JPanel {

    private java.util.List dataModuleList = new ArrayList();

    private FieldRobot robot;

    public RobotDataOutput(FieldRobot robot) {
        this.robot = robot;
        setBackground(Color.WHITE);
        putComponents();
    }

    private void putComponents() {
        setLayout(new GridBagLayout());

        GridBagConstraints gc = new GridBagConstraints();
        gc.insets = new Insets(5, 5, 5, 5);
        gc.weightx = 0.0; gc.weighty = 0.0; gc.fill = GridBagConstraints.NONE; gc.anchor = GridBagConstraints.WEST;
        gc.gridx = 0; gc.gridy = 0; gc.gridwidth = 2; gc.gridheight = 1;
        JLabel transportLabel = new JLabel(robot.getDescription());
        add(transportLabel, gc);

        gc.gridy += 3;
        JLabel sensorsLabel = new JLabel("Sensors: ");
        add(sensorsLabel, gc);

        gc.gridy += 2;
        gc.gridx = 1; gc.gridwidth = 1;

        dataModuleList = new ArrayList();

        for (Iterator iter = robot.getSensors().iterator(); iter.hasNext();) {
            Sensor sensor = (Sensor) iter.next();

            JLabel sensorLabel = new JLabel();
            SensorDataModule module = new SensorDataModule(sensor, sensorLabel);
            dataModuleList.add(module);
            module.update();

            add(sensorLabel, gc);
            gc.gridy++;
        }

        gc.gridy += 1;
        JLabel lcdLabel = new JLabel("");
        LcdDataModule lcdModule = new LcdDataModule(robot, lcdLabel);
        dataModuleList.add(lcdModule);
        add(lcdLabel, gc);
        lcdModule.update();

        gc.gridy += 1;
        JLabel autoLabel = new JLabel("Auto: ");
        add(autoLabel, gc);

        gc.gridy += 2;

        int i = 0;

        for (Iterator iter = robot.getControlSystem().getControlAutomata().iterator(); iter.hasNext();) {
            ControlAutomaton auto = (ControlAutomaton) iter.next();

            JLabel automatonLabel = new JLabel();
            AutomatonDataModule module = new AutomatonDataModule(auto, automatonLabel);
            dataModuleList.add(module);
            module.update();

            add(automatonLabel, gc);

            if (i % 2 == 0) {
                gc.gridx = 2;
            } else {
                gc.gridy++;
                gc.gridx = 1;
            }
            i++;
        }


        updateComponents();
    }

    public void updateComponents() {
        for (Iterator iter = dataModuleList.iterator(); iter.hasNext(); ) {
            DataModule module = (DataModule) iter.next();
            module.update();
        }
    }

    public void paintComponent(Graphics g) {
        updateComponents();
        super.paintComponent(g);

        int w = getWidth();
        int h = getHeight();

        g.setColor(Color.BLACK);
        g.drawLine(0, 0, w - 1, 0);
        g.drawLine(w - 1, 0, w - 1, h - 1);
        g.drawLine(w - 1, h - 1, 0, h - 1);
        g.drawLine(0, h - 1, 0, 0);

    }

    private interface DataModule {
        public void update();
    }

    private class SensorDataModule implements DataModule {
        private Sensor sensor;
        private JLabel label;

        public SensorDataModule(Sensor sensor, JLabel label) {
            this.sensor = sensor;
            this.label = label;
        }

        public void update() {
            label.setText(sensor.getDescription() + ": " + sensor.getValue());
        }
    }

    private class LcdDataModule implements DataModule {
        private FieldRobot robot;
        private JLabel label;

        public LcdDataModule(FieldRobot robot, JLabel label) {
            this.robot = robot;
            this.label = label;
        }

        public void update() {
            label.setText("LCD: " + robot.getLcdMessage());
        }
    }

    private class AutomatonDataModule implements DataModule {
        private ControlAutomaton auto;
        private JLabel label;

        public AutomatonDataModule(ControlAutomaton auto, JLabel label) {
            this.auto = auto;
            this.label = label;
        }

        public void update() {
            label.setText("A" + auto.getId() + ": " + auto.getAutoState());
        }
    }
}