package wheelexp;

import physics.Constants;
import physics.MatrixSolver;

import javax.swing.*;
import java.awt.*;
import java.awt.event.AdjustmentListener;
import java.awt.event.AdjustmentEvent;
import java.awt.event.ItemListener;
import java.awt.event.ItemEvent;
import java.awt.geom.AffineTransform;
import java.text.NumberFormat;

/**
 * @author Boris Yartsev
 */
public class DoubleWheelOneDimTest extends JFrame {
    private BikePhysicsEmulator pe;

    public static final int DEFAULT_WHEEL_WIDTH = 330;
    public static final int DEFAULT_WHEEL_HEIGHT = 200;

    public static final int DEFAULT_CONTROL_WIDTH = 330;
    public static final int DEFAULT_CONTROL_HEIGHT = 150;

    private BikeDrawPanel bdpanel;
    private BikeControlPanel cpanel;

    public DoubleWheelOneDimTest() {
        setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        pe = new BikePhysicsEmulator();
        bdpanel = new BikeDrawPanel();
        bdpanel.setPreferredSize(new Dimension(DEFAULT_WHEEL_WIDTH, DEFAULT_WHEEL_HEIGHT));
        getContentPane().setLayout(new BorderLayout());
        getContentPane().add(bdpanel, BorderLayout.CENTER);


        cpanel = new BikeControlPanel();
        cpanel.setPreferredSize(new Dimension(DEFAULT_CONTROL_WIDTH, DEFAULT_CONTROL_HEIGHT));
        getContentPane().add(cpanel, BorderLayout.SOUTH);

        pack();
        setResizable(false);
        setVisible(true);
    }

    private class BikeControlPanel extends JPanel implements AdjustmentListener, ItemListener {

        //This scrollBar is from th  0 to 1.0
        private JScrollBar frictionScrollBar;
        private JLabel frictionLabel;

        //This scrollBar is from -10 to 10
        private JScrollBar moment1ScrollBar;
        private JLabel moment1Label;

        //This scrollBar is from -10 to 10
        private JScrollBar moment2ScrollBar;
        private JLabel moment2Label;

        //this check box allows to synchronize between two moment bars
        private JCheckBox syncCheckBox;
        private boolean checked = true;

        //this scrollbar sets the place of the mass
        private JScrollBar massCenterScrollBar;
        private JLabel massCenterValueLabel;

        private NumberFormat format;

        private int min = 0;
        private int max = 100;

        private double maxFC = 1;
        private double maxM = 100;


        public BikeControlPanel() {
            format = NumberFormat.getNumberInstance();
            format.setMaximumFractionDigits(2);

            setLayout(new GridBagLayout());
            GridBagConstraints gc = new GridBagConstraints();

            gc.insets = new Insets(5, 5, 5, 5);
            gc.fill = GridBagConstraints.HORIZONTAL;
            gc.gridx = 0;
            gc.gridy = 0;
            gc.gridwidth = 4;
            gc.gridheight = 1;
            gc.weightx = 1;
            gc.weighty = 0;

            frictionScrollBar = new JScrollBar(JScrollBar.HORIZONTAL);
            add(frictionScrollBar, gc);
            frictionScrollBar.setValues((int)((pe.getFrictionCoefficient() / maxFC) * max), 1, min, max + 1);
            frictionScrollBar.repaint();
            frictionScrollBar.addAdjustmentListener(this);

            gc.gridx = 4;
            gc.gridy = 0;
            gc.gridwidth = 4;
            frictionLabel = new JLabel();
            updateFrictionLabel(pe.getFrictionCoefficient());
            add(frictionLabel, gc);

            gc.gridx = 0;
            gc.gridy = 1;
            gc.gridwidth = 4;
            moment1ScrollBar = new JScrollBar(JScrollBar.HORIZONTAL);
            add(moment1ScrollBar, gc);
            moment1ScrollBar.setValues((int) (((pe.getSpinMoment1() + maxM) / maxM) * max / 2), 1, min, max + 1);
            moment1ScrollBar.repaint();
            moment1ScrollBar.addAdjustmentListener(this);

            gc.gridx = 4;
            gc.gridy = 1;
            gc.gridwidth = 4;

            moment1Label = new JLabel();
            updateSpinMoment1Label(pe.getSpinMoment1());
            add(moment1Label, gc);

            gc.gridx = 0;
            gc.gridy = 2;
            gc.gridwidth = 4;
            moment2ScrollBar = new JScrollBar(JScrollBar.HORIZONTAL);
            add(moment2ScrollBar, gc);
            moment2ScrollBar.setValues((int) (((pe.getSpinMoment1() + maxM) / maxM) * max / 2), 1, min, max + 1);
            moment2ScrollBar.repaint();
            moment2ScrollBar.addAdjustmentListener(this);

            gc.gridx = 4;
            gc.gridy = 2;
            gc.gridwidth = 4;

            moment2Label = new JLabel();
            updateSpinMoment2Label(pe.getSpinMoment2());
            add(moment2Label, gc);

            gc.gridx = 0;
            gc.gridy = 3;
            gc.gridwidth = 2;
            syncCheckBox = new JCheckBox("Sync moment", true);
            syncCheckBox.addItemListener(this);
            add(syncCheckBox, gc);
            syncCheckBox.repaint();

            gc.gridx = 0;
            gc.gridy = 4;
            gc.gridwidth = 4;
            massCenterScrollBar = new JScrollBar(JScrollBar.HORIZONTAL);
            add(massCenterScrollBar, gc);
            massCenterScrollBar.setValues((int) ((pe.getNormWeightCenterPosition() + 1) * max / 2), 1, min, max + 1);
            massCenterScrollBar.repaint();
            massCenterScrollBar.addAdjustmentListener(this);

            gc.gridx = 4;
            gc.gridy = 4;
            gc.gridwidth = 4;
            massCenterValueLabel = new JLabel();
            updateCenterLabel(pe.getNormWeightCenterPosition());
            add(massCenterValueLabel, gc);
        }

        private void updateFrictionLabel(double value) {
            String fc = format.format(value);
            frictionLabel.setText("Friction: " + fc);
        }

        private void updateSpinMoment1Label(double value) {
            String fc = format.format(value);
            moment1Label.setText("SpinMoment1: " + fc);
        }

        private void updateSpinMoment2Label(double value) {
            String fc = format.format(value);
            moment2Label.setText("SpinMoment2: " + fc);
        }

        private void updateCenterLabel(double value) {
            String fc = format.format(value);
            massCenterValueLabel.setText("MassCenter: " + fc);
        }

        public void adjustmentValueChanged(AdjustmentEvent e) {
            if (e.getSource() == frictionScrollBar) {
                double fc = ((double) frictionScrollBar.getValue() / max + Constants.EPS) * maxFC;
                updateFrictionLabel(fc);
                pe.setFrictionCoefficient(fc);
            } else

            if (e.getSource() == moment1ScrollBar) {
                double sm = (((double) moment1ScrollBar.getValue() - max / 2) / max + Constants.EPS) * 2 * maxM;
                updateSpinMoment1Label(sm);
                pe.setSpinMoment1(sm);

                if (checked) {
                    updateSpinMoment2Label(sm);
                    pe.setSpinMoment2(sm);
                    moment2ScrollBar.setValue(moment1ScrollBar.getValue());
                    moment2ScrollBar.repaint();
                }
            } else

            if (e.getSource() == moment2ScrollBar) {
                double sm = (((double) moment2ScrollBar.getValue() - max / 2) / max + Constants.EPS) * 2 * maxM;
                updateSpinMoment2Label(sm);
                pe.setSpinMoment2(sm);

                if (checked) {
                    updateSpinMoment2Label(sm);
                    pe.setSpinMoment2(sm);
                    moment1ScrollBar.setValue(moment2ScrollBar.getValue());
                    moment1ScrollBar.repaint();
                }
            } else

            if (e.getSource() == massCenterScrollBar) {
                double sm = (((double) massCenterScrollBar.getValue() - max / 2) / max + Constants.EPS) * 2;
                updateCenterLabel(sm);
                pe.setNormWeightCenterPosition(sm);
            }

        }

        public void itemStateChanged(ItemEvent e) {
            if (e.getSource() == syncCheckBox) {
                checked = (e.getStateChange() == ItemEvent.SELECTED);
            }
        }
    }

    private class BikeDrawPanel extends JPanel {
        private Image buffer;

        private Image wheelImage = Toolkit.getDefaultToolkit().getImage("wheel.png");
        private Image trackImage = Toolkit.getDefaultToolkit().getImage("track.png");
        private Image connectorImage = Toolkit.getDefaultToolkit().getImage("connector.png");

        private AffineTransform trans = new AffineTransform();

        public static final int LEFT_PAD = 50;
        public static final int UPPER_PAD = 50;

        /**
         * Todo refactor the code and remove all of these ugly constants
         * @param g
         */
        public void paintComponent(Graphics g) {
            int w = getSize().width;
            int h = getSize().height;

            if (buffer == null) {
                buffer = createImage(w, h);
            }

            Graphics2D ig = (Graphics2D) buffer.getGraphics();

            //fill everything with the white color
            ig.setColor(Color.WHITE);
            ig.fillRect(0, 0, w, h);


            trans.setToIdentity();
            trans.translate(LEFT_PAD, UPPER_PAD);
            trans.rotate(-pe.getFirstWheelAngle(),
                    wheelImage.getWidth(null) / 2, wheelImage.getHeight(null) / 2);

            ig.drawImage(wheelImage, trans, null);


            trans.setToIdentity();
            trans.translate(LEFT_PAD + wheelImage.getWidth(null) + 22, UPPER_PAD);
            trans.rotate(-pe.getSecondWheelAngle(),
                    wheelImage.getWidth(null) / 2, wheelImage.getHeight(null) / 2);

            ig.drawImage(wheelImage, trans, null);


            trans.setToIdentity();
            trans.translate(LEFT_PAD + wheelImage.getWidth(null) / 2 - 5, UPPER_PAD + wheelImage.getHeight(null) / 2 - 5);
            ig.drawImage(connectorImage, trans, null);

            double cmx = pe.getNormWeightCenterPosition();
            int cmxposx = LEFT_PAD + wheelImage.getWidth(null) / 2 - 11 + (int) ((cmx + 1) * connectorImage.getWidth(null) / 2);
            int cmxposy = UPPER_PAD + wheelImage.getHeight(null) / 2 - 5;

            ig.setColor(Color.RED);
            ig.fillOval(cmxposx, cmxposy, connectorImage.getHeight(null), connectorImage.getHeight(null));

            //the coordinate of the first track 100 is the
            int ftx = (int) Math.round(-100 * pe.getBikePosition());

            ftx = (ftx > 0) ? (ftx % trackImage.getWidth(null)) :
                    (trackImage.getWidth(null) - (-ftx) % trackImage.getWidth(null));
            ftx = ftx - trackImage.getWidth(null);

            int n = w / trackImage.getWidth(null) + 3;

            for (int i = 0; i < n; i++) {
                trans.setToIdentity();
                trans.translate(ftx + i * trackImage.getWidth(null), UPPER_PAD + wheelImage.getWidth(null));
                ig.drawImage(trackImage, trans, null);
            }

            //Drawing the Friction Force 1
            drawForce(ig, false, pe.getNormFF1(), wheelImage.getWidth(null) / 2,
                    LEFT_PAD + wheelImage.getWidth(null) / 2, UPPER_PAD + wheelImage.getHeight(null));

            //Drawing the Friction Force 2
            drawForce(ig, false, pe.getNormFF2(), wheelImage.getWidth(null) / 2,
                    LEFT_PAD + wheelImage.getWidth(null) / 2 + wheelImage.getWidth(null) + 22,
                    UPPER_PAD + wheelImage.getHeight(null));

            //Drawing the reaction force 1
            drawForce(ig, true, pe.getNormRF1(), wheelImage.getHeight(null) / 2,
                    LEFT_PAD + wheelImage.getWidth(null) / 2,
                    UPPER_PAD + wheelImage.getHeight(null));

            //Drawing the reaction force 2
            drawForce(ig, true, pe.getNormRF2(), wheelImage.getHeight(null) / 2,
                    LEFT_PAD + wheelImage.getWidth(null) / 2 + wheelImage.getWidth(null) + 22,
                    UPPER_PAD + wheelImage.getHeight(null));

            g.drawImage(buffer, 0, 0, null);
        }

        private void drawForce(Graphics g, boolean vertical, double normValue, int linearValue, int x, int y) {
            int x1 = 0;
            int x2 = (int) (normValue * linearValue);
            int xmax1 = 0;
            int xmax2 = (int) (Math.signum(normValue) * linearValue);

            if (x2 < x1 || xmax2 < xmax1) {
                int t = x2; x2 = x1; x1 = t;
                t = xmax2; xmax2 = xmax1; xmax1 = t;
            }

            if (vertical) {
                g.setColor(Color.GREEN);
                g.fillRect(x, y - xmax2, 3, xmax2 - xmax1);
                g.setColor(Color.RED);
                g.fillRect(x, y - x2, 3, x2 - x1);
            } else {
                g.setColor(Color.GREEN);
                g.fillRect(x + xmax1, y, xmax2 - xmax1, 3);
                g.setColor(Color.RED);
                g.fillRect(x + x1, y, x2 - x1, 3);
            }
        }
    }

    public class BikePhysicsEmulator {

        /**
         * The calculating state of the system
         */
        private int state;

        public static final int DRIVING_STATE = 1;
        public static final int FLYING_STATE = 2;

        /**
         * The length of the ramp between wheels
         */
        private double rampL = 1.4;

        /**
         * The X coodinate of the bike
         */
        private double bikeX = 0;

        /**
         * The velocity of the bike
         */
        private double bikeV = 0;

        /**
         * The mass of the bike
         */
        private double bikeM = 1000;

        /**
         * The friction coefficient between two surfaces
         */
        private double FC = 0.15;

        /**
         * Parameters, describing the first wheel
         */
        private double wheel1I = 10;
        private double wheel1R = 0.5;
        private double wheel1L = 0.7;

        /**
         * The angle and the velocity of the first wheel
         */
        private double wheel1W = 0;
        private double wheel1A = 0;

        /**
         * The spinning moment of the first wheel
         */
        private double wheel1SM = 0;



        /**
         * Parameters, describing the second wheel
         */
        private double wheel2I = 10;
        private double wheel2R = 0.5;
        private double wheel2L = 0.7;

        /**
         * The angle and the velocity of the second wheel
         */
        private double wheel2W = 0;
        private double wheel2A = 0;


        /**
         * The spinning moment of the second wheel
         */
        private double wheel2SM = 0;


        /**
         * The norm reacltion forces
         */
        private double N1 = 0;
        private double N2 = 0;

        private double FF1 = 0;
        private double FF2 = 0;

        private double FM1 = 0;
        private double FM2 = 0;

        private MatrixSolver solver2;

        public BikePhysicsEmulator() {
            //solver for matrix with 2 variables
            solver2 = new MatrixSolver(2);
            state = DRIVING_STATE;
        }

        public double getFirstWheelAngle() {
            return wheel1A;
        }

        public double getSecondWheelAngle() {
            return wheel2A;
        }

        public double getBikePosition() {
            return bikeX;
        }

        /**
         * Recalculates the coordinates and the speeds of the bike
         * @param delta
         */
        public void updateCoordinates(double delta) {
            if (state == FLYING_STATE) {
                System.out.println("Flying");
                return;
            }

            double wheel1LV = wheel1W * wheel1R + bikeV;
            double wheel2LV = wheel2W * wheel2R + bikeV;

            boolean w1b = Math.abs(wheel1LV) > Constants.EPS;
            boolean w2b = Math.abs(wheel2LV) > Constants.EPS;

            if (w1b && w2b) {
                double ff1dir = -Math.signum(wheel1LV);
                double ff2dir = -Math.signum(wheel2LV);

                double a2 = wheel1L - ff1dir * wheel1R * FC;
                double b2 = -wheel2L - ff2dir * wheel2R * FC;

                solver2.loadCoeffAt(1, 0, 0);
                solver2.loadCoeffAt(1, 0, 1);
                solver2.loadCoeffAt(a2, 1, 0);
                solver2.loadCoeffAt(b2, 1, 1);

                solver2.loadVectorAt(bikeM * Constants.G, 0);
                solver2.loadVectorAt(0, 1);

                N1 = solver2.getAnswer()[0];
                N2 = solver2.getAnswer()[1];

                //if both forces are less than zero, then our car is flying
                //todo this is crooked, but still
                if ((N1 < 0) || (N2 < 0)) {
                    state = FLYING_STATE;
                    return;
                }

                FF1 = FC * N1 * ff1dir;
                FF2 = FC * N2 * ff2dir;
            } else

            if (w1b && !w2b) {

            } else

            if (!w1b && w2b) {

            } else

            if (!w1b && !w2b) {
                double a1 = wheel1I / (bikeM * wheel1R) + wheel1R;
                double b1 = wheel1I / (bikeM * wheel1R);
                double c1 = -wheel1SM;

                double a2 = wheel2I / (bikeM * wheel2R);
                double b2 = wheel2I / (bikeM * wheel2R) + wheel2R;
                double c2 = -wheel2SM;

                solver2.loadCoeffAt(a1, 0, 0);
                solver2.loadCoeffAt(b1, 0, 1);
                solver2.loadCoeffAt(a2, 1, 0);
                solver2.loadCoeffAt(b2, 1, 1);
                solver2.loadVectorAt(c1, 0);
                solver2.loadVectorAt(c2, 1);

                FF1 = solver2.getAnswer()[0];
                FF2 = solver2.getAnswer()[1];

                /**
                 * Solving the system for the moments of the forces of the wheels
                 */
                solver2.loadCoeffAt(1, 0, 0);
                solver2.loadCoeffAt(1, 0, 1);
                solver2.loadCoeffAt(wheel1L, 1, 0);
                solver2.loadCoeffAt(-wheel2L, 1, 1);
                solver2.loadVectorAt(bikeM * Constants.G, 0);
                solver2.loadVectorAt(FF1 * wheel1R + FF2 * wheel2R, 1);

                N1 = solver2.getAnswer()[0];
                N2 = solver2.getAnswer()[1];

                if ((N1 < 0) || (N2 < 0)) {
                    state = FLYING_STATE;
                    return;
                }
            }

            FM1 = FF1 * wheel1R;
            FM2 = FF2 * wheel2R;

            wheel1W += (wheel1SM + FM1) * delta / wheel1I;
            wheel2W += (wheel2SM + FM2) * delta / wheel2I;

            //one object influences the other object with the speed with the same module and oppisite direction
            bikeV += (FF1 + FF2) * delta / bikeM;

            /**
             * Updating the angles of the wheels and the coordinates of the bike
             */
            wheel1A += wheel1W * delta;
            wheel2A += wheel2W * delta;
            bikeX += bikeV * delta;
        }


        /**
         * This part is the beginning of the simple calculations part
         */

        public static final int START_CALC_STATE = 0;
        public static final int NONE_SLIDING_STATE = 1;
        public static final int FIRST_SLIDING_STATE = 2;
        public static final int SECOND_SLIDING_STATE = 3;
        public static final int BOTH_SLIDING_STATE = 4;

        /**
         * todo this is the simple variant
         * We dont care about the moments, that are created by the friction forces -
         * so the car cannot overturn
         * Recalculates the coordinates and the speeds of the bike
         * @param delta
         */
        public void updateCoordinates2(double delta) {
            double wheel1LV = wheel1W * wheel1R + bikeV;
            double wheel2LV = wheel2W * wheel2R + bikeV;

            boolean w1b = Math.abs(wheel1LV) > Constants.EPS;
            boolean w2b = Math.abs(wheel2LV) > Constants.EPS;
            double ff1dir = (w1b) ? -Math.signum(wheel1LV) : 0;
            double ff2dir = (w2b) ? -Math.signum(wheel2LV) : 0;


            z3_calcReactionForces();

            //value -1 means, that the calculation should be performed
            int res = 1;

            while(res != 0) {
                if (w1b && w2b) {
                    z2_2_calcBothSlidingForces(ff1dir, ff2dir);
                } else

                if (!w1b && w2b) {
                    z2_4_calcSecondSlidingForces(ff2dir);
                } else

                if (w1b && !w2b) {
                    z2_3_calcFirstSlidingForces(ff1dir);
                } else

                if (!w1b && !w2b) {
                    z2_1_calcNonSlidingForces();
                }

                res = 0;

                //getting the state of the system
                if (x1_1_firstForcePositiveExceeding()) {
                    w1b = true;
                    ff1dir = 1;
                    res = 1;
                }

                if (x1_2_firstForceNegativeExceeding()) {
                    w1b = true;
                    ff1dir = -1;
                    res = 1;
                }

                if (x2_1_secondForcePositiveExceeding()) {
                    w2b = true;
                    ff2dir = 1;
                    res = 1;
                }

                if (x2_2_secondForceNegativeExceeding()) {
                    w2b = true;
                    ff2dir = 1;
                    res = 1;
                }
            }

            z1_calcNewCoords(delta);
        }

        /**
         * This function by the given forces calculates the new coordinates
         * @param delta
         */
        private void z1_calcNewCoords(double delta) {
            FM1 = FF1 * wheel1R;
            FM2 = FF2 * wheel2R;

            wheel1W += (wheel1SM + FM1) * delta / wheel1I;
            wheel2W += (wheel2SM + FM2) * delta / wheel2I;

            //one object influences the other object with the speed with the same module and oppisite direction
            bikeV += (FF1 + FF2) * delta / bikeM;

            /**
             * Updating the angles of the wheels and the coordinates of the bike
             */
            wheel1A += wheel1W * delta;
            wheel2A += wheel2W * delta;
            bikeX += bikeV * delta;
        }

        private void z2_1_calcNonSlidingForces() {
            double a1 = wheel1I / (bikeM * wheel1R) + wheel1R;
            double b1 = wheel1I / (bikeM * wheel1R);
            double c1 = -wheel1SM;

            double a2 = wheel2I / (bikeM * wheel2R);
            double b2 = wheel2I / (bikeM * wheel2R) + wheel2R;
            double c2 = -wheel2SM;

            solver2.loadCoeffAt(a1, 0, 0);
            solver2.loadCoeffAt(b1, 0, 1);
            solver2.loadCoeffAt(a2, 1, 0);
            solver2.loadCoeffAt(b2, 1, 1);
            solver2.loadVectorAt(c1, 0);
            solver2.loadVectorAt(c2, 1);

            FF1 = solver2.getAnswer()[0];
            FF2 = solver2.getAnswer()[1];
        }

        private void z2_2_calcBothSlidingForces(double ff1dir, double ff2dir) {
            FF1 = FC * N1 * ff1dir;
            FF2 = FC * N2 * ff2dir;
        }

        private void z2_3_calcFirstSlidingForces(double ff1dir) {
            FF1 = FC * N1 * ff1dir;
            double c = wheel2I / (bikeM * wheel2R);

            //now calculating the second force
            FF2 = -(wheel2SM + FF1 * c) / (wheel2R + c);
        }

        private void z2_4_calcSecondSlidingForces(double ff2dir) {
            FF2 = FC * N2 * ff2dir;
            double c = wheel1I / (bikeM * wheel1R);

            //now calculating the second force
            FF1 = -(wheel1SM + FF2 * c) / (wheel1R + c);
        }

        private void z3_calcReactionForces() {
            solver2.loadCoeffAt(1, 0, 0);
            solver2.loadCoeffAt(1, 0, 1);
            solver2.loadCoeffAt(wheel1L, 1, 0);
            solver2.loadCoeffAt(-wheel2L, 1, 1);

            solver2.loadVectorAt(bikeM * Constants.G, 0);
            solver2.loadVectorAt(0, 1);

            N1 = solver2.getAnswer()[0];
            N2 = solver2.getAnswer()[1];
        }

        private boolean x1_1_firstForcePositiveExceeding() {
            return (FF1 > FC * N1);
        }

        private boolean x1_2_firstForceNegativeExceeding() {
            return (FF1 < -FC * N1);
        }

        private boolean x2_1_secondForcePositiveExceeding() {
            return (FF2 > FC * N2);
        }

        private boolean x2_2_secondForceNegativeExceeding() {
            return (FF2 < -FC * N2);
        }


        /**
         * This is the end of the part of simple calculations
         */

        /**
         * Returns normalized value of the mass center - from -1 to 1
         * @return
         */
        public double getNormWeightCenterPosition() {
            return (2 * wheel1L / (rampL) - 1);
        }

        public void setNormWeightCenterPosition(double normValue) {
            double place = rampL * (normValue + 1) / 2;
            wheel1L = place;
            wheel2L = rampL - wheel1L;
        }

        public double getNormRF1() {
            double weightF = bikeM * Constants.G;
            return N1 / weightF;
        }

        public double getNormRF2() {
            double weightF = bikeM * Constants.G;
            return N2 / weightF;
        }

        public double getNormFF1() {
            double FF1MAX = FC * N1;
            return FF1 / FF1MAX;
        }

        public double getNormFF2() {
            double FF2MAX = FC * N2;
            return FF2 / FF2MAX;
        }

        public double getFrictionCoefficient() {
            return FC;
        }

        public void setFrictionCoefficient(double FC) {
            this.FC = FC;
        }

        public double getSpinMoment1() {
            return wheel1SM;
        }

        public double getSpinMoment2() {
            return wheel2SM;
        }

        public void setSpinMoment1(double SM1) {
            wheel1SM = SM1;
        }

        public void setSpinMoment2(double SM2) {
            wheel2SM = SM2;
        }

    }

    public static void main(String[] args) {
        DoubleWheelOneDimTest view = new DoubleWheelOneDimTest();

        while (true) {
            view.pe.updateCoordinates2(Constants.TIME_DELTA);
            view.repaint();
        }
    }
}
