package wheelexp;

import physics.Utilities;
import physics.MatrixSolver;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.geom.AffineTransform;

/**
 * @author Boris Yartsev
 */
public class FourWheelOneDimTest extends JFrame {

    private CarDrawPanel cdp;
    private ControlPanel cp;
    private FourWheelPhysicalEngine fwpe;

    public static final int FRONT_LEFT = 1;
    public static final int FRONT_RIGHT = 2;
    public static final int REAR_LEFT = 3;
    public static final int REAR_RIGHT = 4;
    public static final int CENTER = 5;


    public FourWheelOneDimTest() {
        fwpe = new FourWheelPhysicalEngine();

        setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        cdp = new CarDrawPanel();
        getContentPane().setLayout(new BorderLayout());
        getContentPane().add(cdp, BorderLayout.CENTER);

        cp = new ControlPanel();
        getContentPane().add(cp, BorderLayout.SOUTH);
        pack();
        setResizable(false);
        setVisible(true);
    }

    private class ControlPanel extends JPanel implements ActionListener {
        private JTextField leftFrontMoment = new JTextField("0");
        private JLabel leftFrontLabel = new JLabel("LeftFrontMoment:");
        private JTextField rightFrontMoment = new JTextField("0");
        private JLabel rightFrontLabel = new JLabel("RightFrontMoment:");
        private JTextField leftRearMoment = new JTextField("0");
        private JLabel leftRearLabel = new JLabel("LeftRearMoment:");
        private JTextField rightRearMoment = new JTextField("0");
        private JLabel rightRearLabel = new JLabel("RightRearMoment:");

        private JTextField leftFrontAlpha = new JTextField("0.5");
        private JLabel leftFrontAlphaLabel = new JLabel("LeftFrontAlpha(PI)");
        private JTextField rightFrontAlpha = new JTextField("-0.5");
        private JLabel rightFrontAlphaLabel = new JLabel("RightFrontAlpha(PI)");
        private JTextField leftRearAlpha = new JTextField("0.5");
        private JLabel leftRearAlphaLabel = new JLabel("LeftRearAlpha(PI)");
        private JTextField rightRearAlpha = new JTextField("-0.5");
        private JLabel rightRearAlphaLabel = new JLabel("RightRearAlpha(PI)");

        private JButton apply = new JButton("Apply");

        public ControlPanel() {
            setLayout(new GridBagLayout());
            GridBagConstraints gc = new GridBagConstraints();
            gc.insets = new Insets(5, 5, 5, 5);
            gc.weightx = 1.0;
            gc.weighty = 1.0;
            gc.fill = GridBagConstraints.BOTH;

            gc.gridx = 0; gc.gridy = 0; gc.gridwidth = 1; gc.gridheight = 1;

            add(leftFrontAlphaLabel, gc);
            gc.gridx++;

            add(leftFrontAlpha, gc);
            gc.gridx++;

            add(leftFrontLabel, gc);
            gc.gridx++;

            add(leftFrontMoment, gc);

            gc.gridx = 0;
            gc.gridy++;

            add(rightFrontAlphaLabel, gc);
            gc.gridx++;

            add(rightFrontAlpha, gc);
            gc.gridx++;

            add(rightFrontLabel, gc);
            gc.gridx++;

            add(rightFrontMoment, gc);

            gc.gridx = 0;
            gc.gridy++;

            add(leftRearAlphaLabel, gc);
            gc.gridx++;

            add(leftRearAlpha, gc);
            gc.gridx++;

            add(leftRearLabel, gc);
            gc.gridx++;

            add(leftRearMoment, gc);

            gc.gridx = 0;
            gc.gridy++;

            add(rightRearAlphaLabel, gc);
            gc.gridx++;

            add(rightRearAlpha, gc);
            gc.gridx++;

            add(rightRearLabel, gc);
            gc.gridx++;

            add(rightRearMoment, gc);

            gc.gridx = 0;
            gc.gridy++;

            add(apply, gc);
            apply.addActionListener(this);
        }

        public void actionPerformed(ActionEvent e) {
            fwpe.setMoment(Double.parseDouble(leftFrontMoment.getText()), FRONT_LEFT);
            fwpe.setMoment(Double.parseDouble(rightFrontMoment.getText()), FRONT_RIGHT);
            fwpe.setMoment(Double.parseDouble(leftRearMoment.getText()), REAR_LEFT);
            fwpe.setMoment(Double.parseDouble(rightRearMoment.getText()), REAR_RIGHT);
            fwpe.setWheelAnglePi(Double.parseDouble(leftFrontAlpha.getText()), FRONT_LEFT);
            fwpe.setWheelAnglePi(Double.parseDouble(rightFrontAlpha.getText()), FRONT_RIGHT);
            fwpe.setWheelAnglePi(Double.parseDouble(leftRearAlpha.getText()), REAR_LEFT);
            fwpe.setWheelAnglePi(Double.parseDouble(rightRearAlpha.getText()), REAR_RIGHT);
            fwpe.updateCoordinates();
        }
    }

    private class CarDrawPanel extends JPanel {
        private Image buffer;

        //Affine transpormation, may be needed in future
        //private AffineTransform trans = new AffineTransform();

        public static final int PAD_X = 100;
        public static final int PAD_Y = 100;

        public static final int VEHICLE_WIDTH = 200;
        public static final int VEHICLE_LENGTH = 400;
        public static final int WHEEL_WIDTH = 10;
        public static final int WHEEL_LENGTH = 30;

        public static final int MAX_FORCE_VECTOR_LENGTH = 100;
        public static final int FORCE_VECTOR_WIDTH = 5;
        public final Color forceColor = Color.RED;
        public final Color momentColor = Color.GREEN;

        public Dimension getPreferredSize() {
            return new Dimension(2 * PAD_X + VEHICLE_LENGTH, 2 * PAD_Y + VEHICLE_WIDTH);
        }

        public void paintComponent(Graphics g) {
            int w = getSize().width;
            int h = getSize().height;

            if (buffer == null) {
                buffer = createImage(w, h);
            }

            Graphics2D ig = (Graphics2D) buffer.getGraphics();

            //fill everything with the white color
            ig.setColor(Color.WHITE);
            ig.fillRect(0, 0, w, h);

            drawWheel(ig, FRONT_LEFT, fwpe.getWheelAngle(FRONT_LEFT) - Math.PI / 2);
            drawWheel(ig, FRONT_RIGHT, fwpe.getWheelAngle(FRONT_RIGHT) - Math.PI / 2);
            drawWheel(ig, REAR_LEFT, fwpe.getWheelAngle(REAR_LEFT) - Math.PI / 2);
            drawWheel(ig, REAR_RIGHT, fwpe.getWheelAngle(REAR_RIGHT) - Math.PI / 2);

            drawForce(ig, FRONT_LEFT, fwpe.getForceX(FRONT_LEFT), fwpe.getForceY(FRONT_LEFT), forceColor);

            double mx = Math.cos(fwpe.getWheelAngle(FRONT_LEFT)) * fwpe.getMoment(FRONT_LEFT);
            double my = Math.sin(fwpe.getWheelAngle(FRONT_LEFT)) * fwpe.getMoment(FRONT_LEFT);
            drawForce(ig, FRONT_LEFT, mx, my, momentColor);

            drawForce(ig, FRONT_RIGHT, fwpe.getForceX(FRONT_RIGHT), fwpe.getForceY(FRONT_RIGHT), forceColor);

            mx = Math.cos(fwpe.getWheelAngle(FRONT_RIGHT)) * fwpe.getMoment(FRONT_RIGHT);
            my = Math.sin(fwpe.getWheelAngle(FRONT_RIGHT)) * fwpe.getMoment(FRONT_RIGHT);
            drawForce(ig, FRONT_RIGHT, mx, my, momentColor);

            drawForce(ig, REAR_LEFT, fwpe.getForceX(REAR_LEFT), fwpe.getForceY(REAR_LEFT), forceColor);

            mx = Math.cos(fwpe.getWheelAngle(REAR_LEFT)) * fwpe.getMoment(REAR_LEFT);
            my = Math.sin(fwpe.getWheelAngle(REAR_LEFT)) * fwpe.getMoment(REAR_LEFT);
            drawForce(ig, REAR_LEFT, mx, my, momentColor);

            drawForce(ig, REAR_RIGHT, fwpe.getForceX(REAR_RIGHT), fwpe.getForceY(REAR_RIGHT), forceColor);

            mx = Math.cos(fwpe.getWheelAngle(REAR_RIGHT)) * fwpe.getMoment(REAR_RIGHT);
            my = Math.sin(fwpe.getWheelAngle(REAR_RIGHT)) * fwpe.getMoment(REAR_RIGHT);
            drawForce(ig, REAR_RIGHT, mx, my, momentColor);

            drawForce(ig, CENTER, fwpe.getForceX(CENTER), fwpe.getForceY(CENTER), forceColor);

            g.drawImage(buffer, 0, 0, null);
        }

        //The polygones for drawing the vectors of the forces
        private int[] polyX = new int[4];
        private int[] polyY = new int[4];

        private void drawForce(Graphics2D g, int wheel, double xf, double yf, Color color) {
            int x = PAD_X;
            int y = PAD_Y;

            switch(wheel) {
                case FRONT_LEFT:
                    x += VEHICLE_LENGTH;
                    break;
                case FRONT_RIGHT:
                    x += VEHICLE_LENGTH;
                    y += VEHICLE_WIDTH;
                    break;
                case REAR_LEFT:
                    break;
                case REAR_RIGHT:
                    y += VEHICLE_WIDTH;
                    break;
                case CENTER:
                    x += VEHICLE_LENGTH / 2;
                    y += VEHICLE_WIDTH / 2;
            }

            //converting the force values to the angle
            double alpha = Utilities.convertToAngle(xf, yf);

            double tx = Math.cos(alpha);
            double ty = Math.sin(alpha);
            double normValue = Utilities.convertToRadius(xf, yf);

            polyX[0] = 0;
            polyY[0] = FORCE_VECTOR_WIDTH / 2;

            polyX[1] = 0;
            polyY[1] = -FORCE_VECTOR_WIDTH / 2;

            polyX[2] = (int) normValue;
            polyY[2] = -FORCE_VECTOR_WIDTH / 2;

            polyX[3] = (int) normValue;
            polyY[3] = FORCE_VECTOR_WIDTH / 2;

            for (int i = 0; i < 4; i++) {
                double nx = polyX[i] * tx - polyY[i] * ty;
                double ny = polyX[i] * ty + polyY[i] * tx;
                polyX[i] = (int) (Math.round(nx)) + x;
                polyY[i] = (int) (Math.round(-ny)) + y;
            }

            g.setColor(color);
            g.fillPolygon(polyX, polyY, 4);
        }

        private void drawWheel(Graphics2D g, int wheel, double alpha) {
            int x = PAD_X;
            int y = PAD_Y;

            switch(wheel) {
                case FRONT_LEFT:
                    x += VEHICLE_LENGTH;
                    break;
                case FRONT_RIGHT:
                    x += VEHICLE_LENGTH;
                    y += VEHICLE_WIDTH;
                    break;
                case REAR_LEFT:
                    break;
                case REAR_RIGHT:
                    y += VEHICLE_WIDTH;
                    break;
            }

            double tx = Math.cos(alpha);
            double ty = Math.sin(alpha);

            polyX[0] = WHEEL_LENGTH / 2;
            polyY[0] = WHEEL_WIDTH / 2;

            polyX[1] = WHEEL_LENGTH / 2;
            polyY[1] = -WHEEL_WIDTH / 2;

            polyX[2] = -WHEEL_LENGTH / 2;
            polyY[2] = -WHEEL_WIDTH / 2;

            polyX[3] = -WHEEL_LENGTH / 2;
            polyY[3] = WHEEL_WIDTH / 2;

            for (int i = 0; i < 4; i++) {
                double nx = polyX[i] * tx - polyY[i] * ty;
                double ny = polyX[i] * ty + polyY[i] * tx;
                polyX[i] = (int) (Math.round(nx) + x);
                polyY[i] = (int) (Math.round(-ny) + y);
            }

            g.setColor(Color.BLACK);
            g.fillPolygon(polyX, polyY, 4);
        }

//        private void drawForce(Graphics g, boolean vertical, double convertToRadius, int linearValue, int x, int y) {
//            int x1 = 0;
//            int x2 = (int) (convertToRadius * linearValue);
//            int xmax1 = 0;
//            int xmax2 = (int) (Math.signum(convertToRadius) * linearValue);
//
//            if (x2 < x1 || xmax2 < xmax1) {
//                int t = x2; x2 = x1; x1 = t;
//                t = xmax2; xmax2 = xmax1; xmax1 = t;
//            }
//
//            if (vertical) {
//                g.setColor(Color.GREEN);
//                g.fillRect(x, y - xmax2, 3, xmax2 - xmax1);
//                g.setColor(Color.RED);
//                g.fillRect(x, y - x2, 3, x2 - x1);
//            } else {
//                g.setColor(Color.GREEN);
//                g.fillRect(x + xmax1, y, xmax2 - xmax1, 3);
//                g.setColor(Color.RED);
//                g.fillRect(x + x1, y, x2 - x1, 3);
//            }
//        }
    }


    private class FourWheelPhysicalEngine {
        private double VM = 100; //the mass of the four wheel robot
        private double IM = 1; //the Inertia moment of the robot

        private double X = 0; //the x coordinate of the robot
        private double Y = 0; //the y coordinate of the robot
        private double AL = 0; //the alpha of the robot

        private double VX = 0; //the x speed of the robot
        private double VY = 0; //the y speed of the robot
        private double W = 0; //the angular velocity of the robot

        private double AX = 0; //the x acceleration
        private double AY = 0; //the y acceleration
        private double E = 0; //the angular acceleration

        private int WC = 4; //the number of wheels
        private double WI = 0.1; //the inertia moment of the wheel
        private double WR = 1; //the radius of the wheel

        private double[] FFX = new double[WC]; //the x components of the friction force
        private double[] FFY = new double[WC]; //the y components of the friction force

        private double[] AWh = new double[WC]; //the angle of each wheel
        private double[] WWh = new double[WC]; //the spinning speed of each wheel
        private double[] EWh = new double[WC]; //the acceleration of each wheel

        private double[] M = new double[WC];

        private MatrixSolver solver8; //we need a matrix solver for solving the 8 variable equations

        private double[][] coeff = new double[2 * WC][2 * WC];
        private double[] vector = new double[2 * WC];

        private double[] answer;


        public FourWheelPhysicalEngine() {
            solver8 = new MatrixSolver(2 * WC);
        }

        //the x radiuses of the wheels, when the AL is 0
        private double[] RX = {
            10, 10, -10, -10
        };

        //the y radiuses of the wheels, when the AL is 0
        private double[] RY = {
            10, -10, 10, -10
        };

        //the angles of the wheels, when the AL is 0
        private double[] SA = {
            Math.PI / 2, -Math.PI / 2, Math.PI / 2, -Math.PI / 2
        };

        public void updateCoordinates() {
            prepareTheMatrix();
            solver8.loadMatrix(coeff, vector);

            answer = solver8.getAnswer();

            for (int i = 0; i < WC; i++) {
                FFX[i] = answer[i];
                FFY[i] = answer[i + WC];
            }
        }

        public double getForceX(int wheel) {
            switch(wheel) {
                case FRONT_LEFT:
                    return FFX[0];
                case FRONT_RIGHT:
                    return FFX[1];
                case REAR_LEFT:
                    return FFX[2];
                case REAR_RIGHT:
                    return FFX[3];
                case CENTER:
                    double fcx = 0;
                    for (int i = 0; i < WC; i++) {
                        fcx += FFX[i];
                    }
                    return fcx;
            }

            return 0;
        }


        public double getForceY(int wheel) {
            switch(wheel) {
                case FRONT_LEFT:
                    return FFY[0];
                case FRONT_RIGHT:
                    return FFY[1];
                case REAR_LEFT:
                    return FFY[2];
                case REAR_RIGHT:
                    return FFY[3];
                case CENTER:
                    double fcy = 0;
                    for (int i = 0; i < WC; i++) {
                        fcy += FFY[i];
                    }
                    return fcy;
            }

            return 0;
        }

        public void setMoment(double moment, int wheel) {
            switch(wheel) {
                case FRONT_LEFT:
                    M[0] = moment;
                case FRONT_RIGHT:
                    M[1] = moment;
                case REAR_LEFT:
                    M[2] = moment;
                case REAR_RIGHT:
                    M[3] = moment;
                case CENTER:
            }
        }

        public double getMoment(int wheel) {
            switch(wheel) {
                case FRONT_LEFT:
                    return M[0];
                case FRONT_RIGHT:
                    return M[1];
                case REAR_LEFT:
                    return M[2];
                case REAR_RIGHT:
                    return M[3];
            }
            return 0;
        }

        public void setWheelAnglePi(double piAngle, int wheel) {
            switch(wheel) {
                case FRONT_LEFT:
                    SA[0] = piAngle * Math.PI;
                case FRONT_RIGHT:
                    SA[1] = piAngle * Math.PI;
                case REAR_LEFT:
                    SA[2] = piAngle * Math.PI;
                case REAR_RIGHT:
                    SA[3] = piAngle * Math.PI;
                case CENTER:
            }
        }

        public double getWheelAngle(int wheel) {
            switch(wheel) {
                case FRONT_LEFT:
                    return SA[0];
                case FRONT_RIGHT:
                    return SA[1];
                case REAR_LEFT:
                    return SA[2];
                case REAR_RIGHT:
                    return SA[3];
            }

            return 0;
        }


        double [] rx = new double[WC]; //the values of radiuses of the wheels
        double [] ry = new double[WC];

        double [] tx = new double[WC];
        double [] ty = new double[WC];

        double [] nx = new double[WC];
        double [] ny = new double[WC];

        private void prepareTheMatrix() {

            double whAl;
            for (int i = 0; i < WC; i++) {
                rx[i] = Math.cos(AL) * RX[i] - Math.sin(AL) * RY[i]; //the r'x and r'y vectors
                ry[i] = Math.sin(AL) * RX[i] + Math.cos(AL) * RY[i];
                whAl = AL + SA[i];

                nx[i] = Math.cos(whAl);
                ny[i] = Math.sin(whAl);

                tx[i] = Math.sin(whAl);
                ty[i] = -Math.cos(whAl);
            }

            /**
             * The first index is the number of the equation
             * The second index is the number of the friction component.
             * The first four indexes are for the x components, the second four indexes are for the y components
             */
            int n = 4;

            /**
             * These are the equations for the non sliding through the normal of each wheel
             */
            for (int i = 0; i < n; i++) {
                double scoeff = -nx[i] * ry[i] + ny[i] * rx[i];

                for (int j = 0; j < n; j++) {
                    coeff[i][j] = nx[i] / VM;
                    coeff[i][j] += ry[i] * scoeff / IM;

                    coeff[i][j + n] = ny[i] / VM;
                    coeff[i][j + n] += -rx[i] * scoeff / IM;
                }

                vector[i] = 0;
            }

            for (int i = 0; i < n; i++) {
                double scoeff = -tx[i] * ry[i] + ty[i] * rx[i];

                for (int j = 0; j < n; j++) {
                    coeff[i + n][j] = tx[i] / VM;
                    coeff[i + n][j] += ry[i] * scoeff / IM;

                    coeff[i + n][j + n] = ty[i] / VM;
                    coeff[i + n][j + n] += -rx[i] * scoeff / IM;
                }

                coeff[i + n][i] += (WR * WR * tx[i]) / WI;
                coeff[i + n][i + n] += (WR * WR * ty[i]) / WI;

                vector[i + n] = -M[i] * WR / WI;
            }
        }
    }

    public static void main(String[] args) {
        FourWheelOneDimTest t = new FourWheelOneDimTest();
        while(true) {
            t.repaint();
            try {
                Thread.sleep(100);
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
        }
    }
}
