package wheelexp;

import physics.Constants;

/**
 * @author Boris Yartsev
 */
public class WheelWithElectricEngine {

    /**
     * The itertia moment of the wheel
     */
    private double IM;


    /**
     * The moment of the force, that spins the wheel
     */
    private double spinM;


    /**
     * This is the size of the wheel
     */
    private double wheelRadius;

    /**
     * These are the coordinates of the wheel from the center of the transport robot.
     */
    private double wheelPolarLength;
    private double wheelAngle;

    /**
     * This is the angle of the wheel,
     */
    private double wheelBend;

    /**
     * The spinning speed of the wheel
     */
    private double wSpeed;

    /**
     * This is the direction of the transport
     */
    private double vehicleAngle;

    /**
     * The speeds of the vehicles
     */
    private double vehicleVX;
    private double vehicleVY;
    private double vehicleW;

    /**
     * The friction coefficient
     */
    private double FC;

    /**
     * The reaction force of the wheel
     */
    private double RF;

    /**
     * The X and Y coordinates of the force, a
     */
    private double FX;
    private double FY;

    /**
     * The moment of the force
     */
    private double FM;

    /**
     * This function recalculates the coordinates
     * @param timeDelta
     */
    public void updateCoordinates(double timeDelta) {
        double radiusX = getX();
        double radiusY = getY();

        double VWX = vehicleVX + vehicleW * radiusY;
        double VWY = vehicleVY - vehicleW * radiusX;

        double slCos = Math.cos(vehicleAngle + wheelBend - Math.PI / 2);
        double slSin = Math.sin(vehicleAngle + wheelBend - Math.PI / 2);

        //Calculating the speed of the lower point of the transport
        double VLPX = VWX - vehicleW * wheelRadius * slCos;
        double VLPY = VWY - vehicleW * wheelRadius * slSin;
        double VLP = Math.sqrt(VLPX * VLPX + VLPY * VLPY);

        //The positive value, denoting the maximum value of the friction force
        double maxFF = RF * FC;

        //if there is no sliding with this wheel
        if (VLP < Constants.EPS) {
            //calculating the moments that affect the wheel
            

//            calculating the friction force, that needed to prevent sliding
//            double realFF = -spinM / (wheelRadius + IM / (TM * wheelRad));


        } else {

        }
    }

    public void updateTransportAngle(double transportAngle) {
        this.vehicleAngle = transportAngle;
    }

    public void updateTransportSpeed(double tsx, double tsy, double w) {
        vehicleVX = tsx;
        vehicleVY = tsy;
        vehicleW = w;
    }

    public void updateFrictionParams(double frictionCoeff, double reactionForce) {
        FC = frictionCoeff;
        RF = reactionForce;
    }

    /**
     * Updating the parameters of the force, that affects the wheel
     * @param FX
     * @param FY
     */
    public void updateForce(double FX, double FY) {
        this.FX = FX;
        this.FY = FY;
    }

    /**
     * Updates the moment, that affects the friction moment
     * @param FM
     */
    public void updateMoment(double FM) {
        this.FM = FM;
    }

    /**
     * This function returns the x coordinate of the wheel
     * @return
     */
    public double getX() {
        return wheelPolarLength * Math.cos(vehicleAngle + wheelAngle);
    }

    /**
     * This function returns the y coordinate of the wheel
     * @return
     */
    public double getY() {
        return wheelPolarLength * Math.sin(vehicleAngle + wheelAngle);
    }

}