package wheelexp;

import physics.Constants;

import java.text.NumberFormat;

import wheelexp.Wheel;

/**
 * Boris Yartsev
 */
public class WheelWithEngine implements Wheel {

    /**
     * The itertia moment of the wheel
     */
    private double inertiaMoment;

    /**
     * This is the size of the wheel
     */
    private double wheelSize;

    /**
     * These are the coordinates of the wheel from the center of the transport robot.
     */
    private double wheelRadius;
    private double wheelAngle;

    /**
     * This is the angle of the wheel,
     */
    private double wheelBend;


    /**
     * This is the speed of the wheel
     */
    private double wTorque = 0;

    /**
     * The acceleration of the wheel
     */
    private double accTorque = 0;

    /**
     * The current power of electric engine
     */
    private int power = 0;

    /**
     * The torque moment for the 8 possible values of power
     */
    private double[] torqueM = {
        0,
        0.01,
        0.02,
        0.03,
        0.04,
        0.05,
        0.06,
        0.07
    };

    /**
     * The loss moment of the wheel for different values of power, when the w = maxSpin.
     */
    private double[] lossM = {
        0.02,
        0.04,
        0.06,
        0.08,
        0.10,
        0.12,
        0.14,
        0.16
    };


    /**
     * The maximum spinning speed of the torque
     */
    private double[] maxSpin = {
        0,
        1,
        2,
        3,
        4,
        5,
        6,
        7
    };

    /**
     * This is the angle coordinate of a transport
     */
    private double transportAngle;

    /**
     * The linear speed of a transport
     */
    private double transportSpeedX;
    private double transportSpeedY;

    /**
     * The spinning speed of a transport
     */
    private double transportW;

    /**
     * The friction coefficient and the reaction power
     */
    private double frictionCoeff;
    private double reactionForce;

    /**
     * The values of the frictionForce, that cause the transport to move
     */
    private double frictionForceX;
    private double frictionForceY;
    private double frictionForce;


    private double speedLowerPointX;
    private double speedLowerPointY;
    private double speedLower;

    public WheelWithEngine(double inertiaMoment, double wheelSize, double wheelRadius, double wheelAngle, double wheelBend) {
        this.inertiaMoment = inertiaMoment;
        this.wheelSize = wheelSize;
        this.wheelRadius = wheelRadius;
        this.wheelAngle = wheelAngle;
        this.wheelBend = wheelBend;
    }

    public void recalculateCoords(double timeDelta) {
        double radiusX = getX();
        double radiusY = getY();
        double speedWheelX = transportSpeedX + transportW * radiusY;
        double speedWheelY = transportSpeedY - transportW * radiusX;

        double slCos = Math.cos(transportAngle + wheelBend - Math.PI / 2);
        double slSin = Math.sin(transportAngle + wheelBend - Math.PI / 2);

        //The speed in the lower point of the wheel
        speedLowerPointX = speedWheelX - wTorque * wheelSize * slCos;
        speedLowerPointY = speedWheelY - wTorque * wheelSize * slSin;

        speedLower = Math.sqrt(speedLowerPointX * speedLowerPointX + speedLowerPointY * speedLowerPointY);

        //two situations - if the lower point of the wheel does not move, so there will be one kind of friction.
        //in other case there will be other kind of friction
//        double frictionForce = frictionCoeff * reactionForce;
//
//
//        //got the friction values
//
//        if (speedLower > physics.Constants.EPS) {
//            frictionForceX = -frictionForce * speedLowerPointX / speedLower;
//            frictionForceY = -frictionForce * speedLowerPointY / speedLower;
//        } else {
//            frictionForceX = 0;
//            frictionForceY = 0;
//        }
//
//        double frictionForceWheelTangent = frictionForceX * slCos + frictionForceY * slSin;
//
//        double frictionMoment = frictionForceWheelTangent * wheelSize;
//        double spinMoment = calcCurrentSpinMoment();
//        accTorque = (frictionMoment + spinMoment) / inertiaMoment;
//
//        wTorque += accTorque * timeDelta;
    }

//    private double calcCurrentSpinMoment() {
//        double m;
//        double mult = 1;
//        if (power >= 0) {
//            m = torqueM[power];
////            if (wTorque > maxSpin[power]) {
////                wTorque = maxSpin[power];
////            }
//            //mult = mult + wTorque / maxSpin[power];
//        } else {
//            m = -torqueM[-power];
////            if (-wTorque > maxSpin[-power]) {
////                wTorque = -maxSpin[-power];
////            }
//            //mult = mult + wTorque / maxSpin[-power];
//        }
//        return m * mult;
//    }

/*
    public void recalculateCoords(double timeDelta) {
        double radiusX = getX();
        double radiusY = getY();
        double speedWheelX = transportSpeedX + transportW * radiusY;
        double speedWheelY = transportSpeedY - transportW * radiusX;

        double slCos = Math.cos(transportAngle + wheelBend - Math.PI / 2);
        double slSin = Math.sin(transportAngle + wheelBend - Math.PI / 2);

        //The speed in the lower point of the wheel
        double speedLowerPointX = speedWheelX - wTorque * wheelSize * slCos;
        double speedLowerPointY = speedWheelY - wTorque * wheelSize * slSin;


        //todo temp
        this.speedLowerPointX = speedLowerPointX;
        this.speedLowerPointY = speedLowerPointY;
        //todo end temp

        double speedLower = Math.sqrt(speedLowerPointX * speedLowerPointX + speedLowerPointY * speedLowerPointY);
        double frictionForce = frictionCoeff * reactionForce;


        //got the friction values

        if (speedLower > physics.Constants.EPS) {
            frictionForceX = -frictionForce * speedLowerPointX / speedLower;
            frictionForceY = -frictionForce * speedLowerPointY / speedLower;
        } else {
            frictionForceX = 0;
            frictionForceY = 0;
        }

        double frictionForceWheelTangent = frictionForceX * slCos + frictionForceY * slSin;

        double frictionMoment = frictionForceWheelTangent * wheelSize;
        double spinMoment = calcCurrentSpinMoment();
        accTorque = (frictionMoment + spinMoment) / inertiaMoment;

        wTorque += accTorque * timeDelta;
    }

    private double calcCurrentSpinMoment() {
        double m;
        double mult = 1;
        if (power >= 0) {
            m = torqueM[power];
//            if (wTorque > maxSpin[power]) {
//                wTorque = maxSpin[power];
//            }
            //mult = mult + wTorque / maxSpin[power];
        } else {
            m = -torqueM[-power];
//            if (-wTorque > maxSpin[-power]) {
//                wTorque = -maxSpin[-power];
//            }
            //mult = mult + wTorque / maxSpin[-power];
        }
        return m * mult;
    }
*/
    public void updateTransportAngle(double transportAngle) {
        this.transportAngle = transportAngle;
    }

    public void updateTransportSpeed(double tsx, double tsy, double w) {
        transportSpeedX = tsx;
        transportSpeedY = tsy;
        transportW = w;
    }

    public void updateFrictionParams(double frictionCoeff, double reactionForce) {
        this.frictionCoeff = frictionCoeff;
        this.reactionForce = reactionForce;
    }

    public void setWTorque(double wTorque) {
        this.wTorque = wTorque;
    }

    public double getFrictionForce() {
        return frictionForce;
    }

    public double getX() {
        return wheelRadius * Math.cos(transportAngle + wheelAngle);
    }

    public double getY() {
        return wheelRadius * Math.sin(transportAngle + wheelAngle);
    }

    /**
     * This method not only sets the power, but also updates the value of the spinning speed of the wheel
     * @param power
     */
    public void setPower(int power) {
        this.power = power;
        wTorque = maxSpin[Math.abs(power)];

        if (power < 0) {
            wTorque = -wTorque;
        }
    }

    public boolean isFrictionQuiet() {
        return speedLower < Constants.EPS;
    }


    public String toString() {
        String outData = "";
        NumberFormat format = NumberFormat.getNumberInstance();
        format.setMaximumFractionDigits(4);

        outData += "Angle: " + format.format(wheelAngle / Math.PI);
        outData += " Torque: " + format.format(wTorque);
        outData += " Power: " + power;
        outData += " MaxSpin: " + format.format(wTorque / maxSpin[Math.abs(power)]);
        outData += " LSX: " + format.format(speedLowerPointX);
        outData += " LSY: " + format.format(speedLowerPointY);
        outData += " FFX: " + format.format(frictionForceX);
        outData += " FFY: " + format.format(frictionForceY);

        return outData;
    }
}